@@ -1,13 +1,17 @@
 set -eu
 
-function innobackupex()
-{
-    run_cmd $IB_BIN $IB_ARGS "$@"
-}
-
 function xtrabackup()
 {
+  if [ -z ${WITH_RR+x} ]; then
     run_cmd $XB_BIN $XB_ARGS "$@"
+  else
+    run_cmd rr $XB_BIN $XB_ARGS "$@"
+  fi
+}
+
+function rr_xtrabackup()
+{
+  run_cmd rr $XB_BIN $XB_ARGS "$@"
 }
 
 function mysql()
@@ -41,9 +45,14 @@ function call_mysql_install_db()
 
         cd $MYSQL_BASEDIR
 
-        if ! $MYSQL_INSTALL_DB --defaults-file=${MYSQLD_VARDIR}/my.cnf \
-            --basedir=${MYSQL_BASEDIR} \
-            ${MYSQLD_EXTRA_ARGS}
+        CLIENT_VERSION_STRING=`${MYSQL} --version`
+
+        INSTALL_CMD="${MYSQLD} \
+        --defaults-file=${MYSQLD_VARDIR}/my.cnf \
+        --basedir=${MYSQL_BASEDIR} \
+        --initialize-insecure"
+
+        if ! ${INSTALL_CMD} ${MYSQLD_EXTRA_ARGS}
         then
             vlog "mysql_install_db failed. Server log (if exists):"
             vlog "----------------"
@@ -61,7 +70,7 @@ function call_mysql_install_db()
 function mysql_ping()
 {
     local pid=$1
-    local attempts=60
+    local attempts=${MYSQLD_START_TIMEOUT:-200}
     local i
 
     for ((i=1; i<=attempts; i++))
@@ -109,31 +118,31 @@ function run_cmd_expect_failure()
 
 function load_sakila()
 {
-vlog "Loading sakila"
-${MYSQL} ${MYSQL_ARGS} -e "create database sakila"
-vlog "Loading sakila scheme"
-${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-schema.sql
-vlog "Loading sakila data"
-${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-data.sql
+    vlog "Loading sakila"
+    ${MYSQL} ${MYSQL_ARGS} -e "create database sakila"
+    vlog "Loading sakila scheme"
+    ${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-schema.sql
+    vlog "Loading sakila data"
+    ${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-data.sql
 }
 
 function load_dbase_schema()
 {
-vlog "Loading $1 database schema"
-${MYSQL} ${MYSQL_ARGS} -e "create database $1"
-${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-schema.sql
+    vlog "Loading $1 database schema"
+    ${MYSQL} ${MYSQL_ARGS} -e "create database $1"
+    ${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-schema.sql
 }
 
 function load_dbase_data()
 {
-vlog "Loading $1 database data"
-${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-data.sql
+    vlog "Loading $1 database data"
+    ${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-data.sql
 }
 
 function drop_dbase()
 {
-  vlog "Dropping database $1"
-  run_cmd ${MYSQL} ${MYSQL_ARGS} -e "DROP DATABASE $1"
+    vlog "Dropping database $1"
+    run_cmd ${MYSQL} ${MYSQL_ARGS} -e "DROP DATABASE $1"
 }
 
 ########################################################################
@@ -198,6 +207,21 @@ function init_server_variables()
     SRV_MYSQLD_SOCKET[$id]=`mktemp -t mysql.sock.XXXXXX`
 }
 
+########################################################################
+# Initialize group replication variables such as datadir, tmpdir, etc. and store
+# them with the specified index in SRV_MYSQLD_* arrays to be used by
+# switch_server() later
+########################################################################
+function bulk_init_gr_variables()
+{
+  local number_of_servers=$1
+  for ((i=1; i<=number_of_servers; i++))
+  do
+    init_server_variables $i
+    SRV_MYSQLD_GR_PORT[$i]=`get_free_port $i`
+  done
+}
+
 ########################################################################
 # Reset server variables
 ########################################################################
@@ -219,6 +243,11 @@ function reset_server_variables()
     SRV_MYSQLD_ERRFILE[$id]=
     SRV_MYSQLD_PORT[$id]=
     SRV_MYSQLD_SOCKET[$id]=
+    # Group Replication
+    if [[ ${SRV_MYSQLD_GR_PORT:-x} != "x" ]];
+    then
+      SRV_MYSQLD_GR_PORT[$id]=
+    fi
 }
 
 ##########################################################################
@@ -239,13 +268,12 @@ function switch_server()
 
     MYSQL_ARGS="--defaults-file=$MYSQLD_VARDIR/my.cnf "
     MYSQLD_ARGS="--defaults-file=$MYSQLD_VARDIR/my.cnf ${MYSQLD_EXTRA_ARGS}"
+    MYSQLDUMP_ARGS=
     if [ "`whoami`" = "root" ]
     then
 	MYSQLD_ARGS="$MYSQLD_ARGS --user=root"
     fi
 
-    IB_ARGS="--defaults-file=$MYSQLD_VARDIR/my.cnf \
---no-version-check ${IB_EXTRA_OPTS:-}"
     XB_ARGS="--defaults-file=$MYSQLD_VARDIR/my.cnf \
 --no-version-check ${XB_EXTRA_OPTS:-}"
 
@@ -254,6 +282,12 @@ function switch_server()
     mysql_datadir="$MYSQLD_DATADIR"
     mysql_tmpdir="$MYSQLD_TMPDIR"
     mysql_socket="$MYSQLD_SOCKET"
+
+    # Group Replication
+    if [[ ${SRV_MYSQLD_GR_PORT:-x} != "x" ]];
+    then
+      MYSQLD_GR_PORT=${SRV_MYSQLD_GR_PORT[$id]}
+    fi
 }
 
 ########################################################################
@@ -262,15 +296,30 @@ function switch_server()
 function start_server_with_id()
 {
     local id=$1
+    shift
+    if [[ "$#" -ge 1 ]];
+    then
+      if [[ "$1" = "gr" ]] || [[ "$1" = "standalone" ]];
+      then
+        local type=$1
+        shift
+      fi
+    fi
+    if [[ ${type:-x} = "x" ]];
+    then
+      type="standalone"
+    fi
     local attempts=0
     local max_attempts=5
-    shift
 
     vlog "Starting server with id=$id..."
 
     while true
     do
-        init_server_variables $id
+        if [[ "${type}" = "standalone" ]];
+        then
+          init_server_variables $id
+        fi
         switch_server $id
 
         if [ ! -d "$MYSQLD_VARDIR" ]
@@ -300,12 +349,16 @@ basedir=${MYSQL_BASEDIR}
 datadir=${MYSQLD_DATADIR}
 tmpdir=${MYSQLD_TMPDIR}
 log-error=${MYSQLD_ERRFILE}
+log-error-verbosity=3
 log-bin=mysql-bin
 relay-log=mysql-relay-bin
 pid-file=${MYSQLD_PIDFILE}
 replicate-ignore-db=mysql
+replicate-ignore-db=performance_schema
+replicate-ignore-db=sys
 innodb_log_file_size=48M
 ${MYSQLD_EXTRA_MY_CNF_OPTS:-}
+#core-file
 
 [client]
 socket=${MYSQLD_SOCKET}
@@ -315,6 +368,8 @@ user=root
 ${XB_EXTRA_MY_CNF_OPTS:-}
 EOF
 
+        local new_instance
+
         # Create datadir and call mysql_install_db if it doesn't exist
         if [ ! -d "$MYSQLD_DATADIR" ]
         then
@@ -323,14 +378,23 @@ EOF
 	    # Reserve 900 series for SST nodes
             if [[ $id -lt 900 ]];then
                 call_mysql_install_db
-            else 
+            else
                 vlog "Skiping mysql_install_db of node $id for SST"
             fi
+            new_instance=yes
+        else
+            new_instance=no
         fi
 
         # Start the server
-        echo "Starting ${MYSQLD} ${MYSQLD_ARGS} $* "
-        ${MYSQLD} ${MYSQLD_ARGS} $* &
+        if [ -z ${WITH_RR+x} ]; then
+          echo "Starting ${MYSQLD} ${MYSQLD_ARGS} $* "
+          ${MYSQLD} ${MYSQLD_ARGS} $* &
+        else
+          echo "Starting rr ${MYSQLD} ${MYSQLD_ARGS} $* "
+          rr ${MYSQLD} ${MYSQLD_ARGS} $* &
+        fi
+
         if ! mysql_ping $!
         then
             if grep "another mysqld server running on port" $MYSQLD_ERRFILE
@@ -355,12 +419,16 @@ EOF
         fi
     done
 
-     vlog "Server with id=$id has been started on port $MYSQLD_PORT, \
+    vlog "Server with id=$id has been started on port $MYSQLD_PORT, \
 socket $MYSQLD_SOCKET"
+
+    if [ $new_instance = yes ] && [ "${type}" = "standalone" ] ; then
+        ${MYSQL} ${MYSQL_ARGS} -e "CREATE DATABASE IF NOT EXISTS test"
+    fi
 }
 
 ########################################################################
-# Stop server with the id specified as the first argument.  The server 
+# Stop server with the id specified as the first argument.  The server
 # is stopped in the fastest possible way.
 ########################################################################
 function stop_server_with_id()
@@ -392,7 +460,7 @@ function stop_server_with_id()
 ########################################################################
 function start_server()
 {
-    start_server_with_id 1 $*
+    start_server_with_id 1 'standalone' $*
 }
 
 ########################################################################
@@ -434,6 +502,98 @@ function shutdown_server()
     shutdown_server_with_id 1
 }
 
+########################################################################
+# Start GR cluster
+########################################################################
+function start_group_replication_cluster()
+{
+  local number_of_nodes=$1
+  shift
+  if [[ -z  $number_of_nodes ]];
+  then
+    number_of_nodes=2
+  fi
+  bulk_init_gr_variables ${number_of_nodes}
+  for ((i=1; i<=number_of_nodes; i++))
+  do
+    switch_server ${i}
+    GR_CLUSTER_ADDRESS=""
+    for SRV in "${SRV_MYSQLD_GR_PORT[@]}"
+    do
+      GR_CLUSTER_ADDRESS="${GR_CLUSTER_ADDRESS}127.0.0.1:${SRV},"
+    done
+    MYSQLD_EXTRA_MY_CNF_OPTS="
+gtid_mode=ON
+enforce_gtid_consistency=ON
+plugin_load_add='group_replication.so'
+loose-group_replication_group_name=\"aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa\"
+loose-group_replication_start_on_boot=off
+loose-group_replication_bootstrap_group= off
+loose-group_replication_recovery_use_ssl=ON
+loose-group_replication_recovery_get_public_key=ON
+loose-group_replication_local_address=\"127.0.0.1:${SRV_MYSQLD_GR_PORT[$i]}\"
+loose-group_replication_group_seeds= \"${GR_CLUSTER_ADDRESS%?}\""
+    if [ "$MYSQL_DEBUG_MODE" = "on" ]; then
+      MYSQLD_EXTRA_MY_CNF_OPTS="
+${MYSQLD_EXTRA_MY_CNF_OPTS}
+plugin_dir=${MYSQL_BASEDIR}/lib/plugin/debug"
+    fi
+    start_server_with_id ${i} 'gr' $*
+
+    # config GR User
+    ${MYSQL} ${MYSQL_ARGS} -e "SET SQL_LOG_BIN=0;
+    CREATE USER IF NOT EXISTS rpl_user@'localhost' IDENTIFIED BY 'password' REQUIRE SSL;
+    GRANT REPLICATION SLAVE ON *.* TO rpl_user@'localhost';
+    GRANT BACKUP_ADMIN ON *.* TO rpl_user@'localhost';
+    CREATE USER IF NOT EXISTS rpl_user@'%' IDENTIFIED BY 'password' REQUIRE SSL;
+    GRANT REPLICATION SLAVE ON *.* TO rpl_user@'%';
+    GRANT BACKUP_ADMIN ON *.* TO rpl_user@'%';
+    FLUSH PRIVILEGES;
+    SET SQL_LOG_BIN=1;"
+    ${MYSQL} ${MYSQL_ARGS} -e "RESET SLAVE; CHANGE MASTER TO MASTER_USER='rpl_user', MASTER_PASSWORD='password' FOR CHANNEL 'group_replication_recovery';"
+
+    # Start/Bootstrap cluster
+    if [[ ${i} -eq 1 ]];
+    then
+      $MYSQL ${MYSQL_ARGS} -e "SET GLOBAL group_replication_bootstrap_group=ON;
+        START GROUP_REPLICATION USER='rpl_user', PASSWORD='password';
+        SET GLOBAL group_replication_bootstrap_group=OFF;"
+    else
+      $MYSQL ${MYSQL_ARGS} -e "START GROUP_REPLICATION USER='rpl_user', PASSWORD='password';"
+    fi
+  done
+  check_gr_health $number_of_nodes
+  switch_server 1
+  $MYSQL ${MYSQL_ARGS} -e "CREATE DATABASE IF NOT EXISTS test"
+}
+
+########################################################################
+# Check health of GR cluster
+########################################################################
+function check_gr_health()
+{
+  local number_of_nodes=$1
+  switch_server 1
+  for ((i=1; i<=30; i++))
+  do
+    ONLINE_NODES=$($MYSQL ${MYSQL_ARGS} -NBe "SELECT COUNT(*) FROM performance_schema.replication_group_members WHERE MEMBER_STATE='ONLINE'")
+    if [[ ${ONLINE_NODES} = ${number_of_nodes} ]];
+    then
+      return 0;
+    fi
+    sleep 2;
+  done
+  $MYSQL ${MYSQL_ARGS} -NBe "SELECT * FROM performance_schema.replication_group_members WHERE MEMBER_STATE='ONLINE'"
+  for ((i=1; i <=${number_of_nodes}; i++))
+  {
+    echo "======= Error log from node ${i}"
+    cat ${SRV_MYSQLD_ERRFILE[$i]}
+    echo "================================"
+  }
+
+  die "Not all GR members are online. Aborting"
+}
+
 ########################################################################
 # Force a checkpoint for a server specified with the first argument
 ########################################################################
@@ -447,7 +607,7 @@ function force_checkpoint_with_server_id()
     vlog "Forcing a checkpoint for server #$id"
 
     shutdown_server_with_id $id
-    start_server_with_id $id $*
+    start_server_with_id $id 'standalone' $*
 }
 
 ########################################################################
@@ -611,12 +771,40 @@ function checksum_table()
     $MYSQL $MYSQL_ARGS -Ns -e "CHECKSUM TABLE $2 EXTENDED" $1 | awk {'print $2'}
 }
 
+########################################################################
+# Prints checksum for a given table.
+# Takes database, table name and the list of columns
+########################################################################
+function checksum_table_columns()
+{
+    local db_name=$1
+    local table_name=$2
+    local old_ifs="$IFS"
+    local col_list
+
+    shift ; shift ; IFS="," ; col_list="$*" ; IFS="$old_ifs"
+
+    $MYSQL $MYSQL_ARGS -Ns ${db_name} <<EOF | sed -n 2p
+set @crc := '', @cnt := 0;
+
+select min(least(
+      length(@crc := sha1(concat(
+         @crc,
+         sha1(concat_ws('#', ${col_list}))))),
+      @cnt := @cnt + 1
+   )) as discard
+from ${table_name} use index(PRIMARY);
+
+select concat_ws(":", @cnt, @crc);
+EOF
+}
+
 ##########################################################################
 # Dumps a given database using mysqldump                                 #
 ##########################################################################
 function record_db_state()
 {
-    $MYSQLDUMP $MYSQL_ARGS -t --compact --skip-extended-insert \
+    $MYSQLDUMP $MYSQL_ARGS $MYSQLDUMP_ARGS -t --compact --skip-extended-insert \
         $1 >"$topdir/tmp/$1_old.sql"
 }
 
@@ -627,7 +815,7 @@ function record_db_state()
 ##########################################################################
 function verify_db_state()
 {
-    $MYSQLDUMP $MYSQL_ARGS -t --compact --skip-extended-insert \
+    $MYSQLDUMP $MYSQL_ARGS $MYSQLDUMP_ARGS -t --compact --skip-extended-insert \
         $1 >"$topdir/tmp/$1_new.sql"
     diff -u "$topdir/tmp/$1_old.sql" "$topdir/tmp/$1_new.sql"
 }
@@ -648,37 +836,89 @@ function egrep()
     return ${PIPESTATUS[0]}
 }
 
+########################################################################
+# Run grep against a file using a pattern and exit in case occurencies
+# don't match
+########################################################################
+function grep_count()
+{
+  local pattern=$1
+  local file=$2
+  local expect=$3
+  local occurencies=$(grep -c "${pattern}" ${file})
+  if [[ $occurencies -ne $expect ]];
+  then
+    vlog "grep_count expected ${expect} for pattern ${pattern} but found ${occurencies}"
+    exit -1
+  fi
+}
+
 ####################################################
 # Helper functions for testing incremental backups #
 ####################################################
 function check_full_scan_inc_backup()
 {
-    local xb_performed_bmp_inc_backup="xtrabackup: using the full scan for incremental backup"
-    local xb_performed_full_scan_inc_backup="xtrabackup: using the changed page bitmap"
+    local xb_performed_full_scan_inc_backup="using the full scan for incremental backup"
+    local xb_performed_bitmap_backup="using the changed page bitmap"
+    local xb_performed_pagetracking_inc_backup="using the pagetracking"
 
-    if ! grep -q "$xb_performed_bmp_inc_backup" $OUTFILE ;
+    if ! grep -q "$xb_performed_full_scan_inc_backup" $OUTFILE ;
     then
         vlog "xtrabackup did not perform a full scan for the incremental backup."
         exit -1
     fi
-    if grep -q "$xb_performed_full_scan_inc_backup" $OUTFILE ;
+    if grep -q "$xb_performed_bitmap_backup" $OUTFILE ;
     then
         vlog "xtrabackup appeared to use bitmaps instead of full scan for the incremental backup."
         exit -1
     fi
+    if grep -q "$xb_performed_pagetracking_inc_backup" $OUTFILE ;
+    then
+        vlog "xtrabackup appeared to use pagtracking instead of full scan for the incremental backup."
+        exit -1
+    fi
 }
 
 function check_bitmap_inc_backup()
 {
-    local xb_performed_bmp_inc_backup="xtrabackup: using the full scan for incremental backup"
-    local xb_performed_full_scan_inc_backup="xtrabackup: using the changed page bitmap"
+    local xb_performed_full_scan_inc_backup="using the full scan for incremental backup"
+    local xb_performed_bitmap_backup="using the changed page bitmap"
+    local xb_performed_pagetracking_inc_backup="using the pagetracking"
 
-    if ! grep -q "$xb_performed_full_scan_inc_backup" $OUTFILE ;
+    if ! grep -q "$xb_performed_bitmap_backup" $OUTFILE ;
     then
         vlog "xtrabackup did not use bitmaps for the incremental backup."
         exit -1
     fi
-    if grep -q "$xb_performed_bmp_inc_backup" $OUTFILE ;
+    if grep -q "$xb_performed_full_scan_inc_backup" $OUTFILE ;
+    then
+        vlog "xtrabackup used a full scan instead of bitmaps for the incremental backup."
+        exit -1
+    fi
+    if grep -q "$xb_performed_pagetracking_inc_backup" $OUTFILE ;
+    then
+        vlog "xtrabackup appeared to use pagtracking instead of full scan for the incremental backup."
+        exit -1
+    fi
+}
+
+function check_pagetracking_inc_backup()
+{
+    local xb_performed_full_scan_inc_backup="using the full scan for incremental backup"
+    local xb_performed_bitmap_backup="using the changed page bitmap"
+    local xb_performed_pagetracking_inc_backup="Using pagetracking feature for incremental backup"
+
+    if ! grep -q "$xb_performed_pagetracking_inc_backup" $OUTFILE ;
+    then
+        vlog "xtrabackup did not use pagtracking for the incremental backup."
+        exit -1
+    fi
+    if grep -q "$xb_performed_bitmap_backup" $OUTFILE ;
+    then
+        vlog "xtrabackup used bitmaps for the incremental backup."
+        exit -1
+    fi
+    if grep -q "$xb_performed_full_scan_inc_backup" $OUTFILE ;
     then
         vlog "xtrabackup used a full scan instead of bitmaps for the incremental backup."
         exit -1
@@ -744,6 +984,26 @@ $MYSQL_VERSION_MINOR $MYSQL_VERSION_PATCH`
     [[ $server_str > $version_str ]]
 }
 
+#########################################################################
+# Return 0 if server is debug version
+########################################################################
+function is_debug_server()
+{
+
+  [[ $MYSQL_VERSION =~ .*debug ]]
+
+}
+
+#########################################################################
+# Requires server debug version
+########################################################################
+function require_debug_server()
+{
+    if ! is_debug_server; then
+        skip_test "Requires debug server version"
+    fi
+}
+
 #########################################################################
 # Require a server version higher than the first argument
 ########################################################################
@@ -753,6 +1013,16 @@ function require_server_version_higher_than()
         skip_test "Requires server version higher than $1"
 }
 
+#########################################################################
+# Requires debug pxb version
+########################################################################
+function require_debug_pxb_version()
+{
+    if ! $XB_BIN --help 2>&1 | grep -q debug.=name; then
+        skip_test "Requires debug build"
+    fi
+}
+
 ########################################################################
 # Return 0 if the server version is lower than the first argument
 #########################################################################
@@ -823,6 +1093,121 @@ function require_qpress()
     fi
 }
 
+########################################################################
+# Skip the test if qpress binary is not available
+########################################################################
+function require_lz4()
+{
+    if ! which lz4 > /dev/null 2>&1 ; then
+        skip_test "Requires lz4 to be installed"
+    fi
+}
+
+function require_zstd()
+{
+    if ! which zstd > /dev/null 2>&1 ; then
+        skip_test "Requires zstd to be installed"
+    fi
+}
+
+function require_tokudb()
+{
+    if ! [ -a $(dirname ${MYSQLD})/../lib/mysql/plugin/ha_tokudb.so ]; then
+        skip_test "Requires TokuDB"
+    fi
+
+    # Modified piece from ps_tokudb_admin
+    local SCRIPT_PWD=$(dirname ${MYSQLD})
+    LIBJEMALLOC_LOCATION=""
+
+    # Check location for libjemalloc.so.1
+    printf "Checking location of jemalloc library ...\n"
+    for libjemall in "${SCRIPT_PWD}/lib/mysql" "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
+      if [ -r "$libjemall/libjemalloc.so.1" ]; then
+	LIBJEMALLOC_LOCATION="$libjemall/libjemalloc.so.1"
+	break
+      fi
+    done
+    if [ -z $LIBJEMALLOC_LOCATION ]; then
+      vlog "ERROR: Cannot find libjemalloc.so.1 library. Make sure you have libjemalloc1 on debian|ubuntu or jemalloc on centos package installed.\n\n";
+      skip_test "Requires jemalloc for TokuDB"
+    else
+      vlog "INFO: Using jemalloc library from $LIBJEMALLOC_LOCATION\n\n";
+    fi
+}
+
+########################################################################
+# Return 0 if the xtrabackup has AddressSanitizer support
+########################################################################
+function is_asan()
+{
+    ldd $XB_BIN | grep -q libasan
+    return $?
+}
+
+#########################################################################
+# Skip test if xtrabackup has AddressSanitizer support
+########################################################################
+function skip_if_asan()
+{
+    if is_asan; then
+        skip_test "Incompatible with AddressSanitizer"
+    fi
+}
+
+##############################################################################
+# Start a server with TokuDB plugins loaded and enabled.
+# Server id is 1, any arguments are passsed to the mysqld.
+##############################################################################
+function start_server_with_tokudb()
+{
+    local OLD_LD_PRELOAD=${LD_PRELOAD:-""}
+
+    LD_PRELOAD="$LIBJEMALLOC_LOCATION:$OLD_LD_PRELOAD" TOKU_HUGE_PAGES_OK=1 \
+    start_server $* --plugin-load-add="\
+tokudb=ha_tokudb.so:\
+tokudb_file_map=ha_tokudb.so:\
+tokudb_fractal_tree_info=ha_tokudb.so:\
+tokudb_fractal_tree_block_map=ha_tokudb.so:\
+tokudb_trx=ha_tokudb.so:\
+tokudb_locks=ha_tokudb.so:\
+tokudb_lock_waits=ha_tokudb.so:\
+tokudb_background_job_status=ha_tokudb.so"
+}
+
+function require_rocksdb()
+{
+   if ! test -a $(dirname ${MYSQLD})/../lib/plugin/ha_rocksdb.so ; then
+        skip_test "Requires RocksDB"
+   fi
+}
+
+function init_rocksdb()
+{
+    mysql <<EOF
+INSTALL PLUGIN ROCKSDB SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_CFSTATS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_DBSTATS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_PERF_CONTEXT SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_PERF_CONTEXT_GLOBAL SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_CF_OPTIONS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_GLOBAL_INFO SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_COMPACTION_STATS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_DDL SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_INDEX_FILE_MAP SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_LOCKS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_TRX SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_DEADLOCK SONAME 'ha_rocksdb.so';
+EOF
+}
+
+function require_debug_sync()
+{
+    if ! $XB_BIN --help 2>&1 | grep -q debug-sync; then
+        skip_test "Requires --debug-sync support"
+    fi
+}
+
 ##############################################################################
 # Execute a multi-row INSERT into a specified table.
 #
@@ -957,5 +1342,142 @@ function is_64bit()
     uname -m 2>&1 | grep 'x86_64'
 }
 
+# Return number of dirty pages
+########################################################################
+function innodb_n_dirty_pages()
+{
+    result=$( $MYSQL $MYSQL_ARGS -se \
+        "SHOW STATUS LIKE 'innodb_buffer_pool_pages_dirty'" | \
+        awk '{ print $2 }' )
+    echo "Dirty pages left $result"
+    return $result
+}
+
+# Wait for InnoDB to flush all dirty pages
+########################################################################
+function innodb_wait_for_flush_all()
+{
+    while ! innodb_n_dirty_pages ; do
+        sleep 1
+    done
+
+    flushed_lsn=`innodb_flushed_lsn`
+    while [ `innodb_checkpoint_lsn` -lt "$flushed_lsn" ] ; do
+      sleep 1
+    done
+}
+
+# Return current LSN
+########################################################################
+function innodb_lsn()
+{
+    ${MYSQL} ${MYSQL_ARGS} -e "SHOW ENGINE InnoDB STATUS\G" | \
+        grep "Log sequence number" | awk '{ print $4 }'
+}
+
+# Return flushed LSN
+########################################################################
+function innodb_flushed_lsn()
+{
+    ${MYSQL} ${MYSQL_ARGS} -e "SHOW ENGINE InnoDB STATUS\G" | \
+        grep "Log flushed up to" | awk '{ print $5 }'
+}
+
+# Return checkpoint LSN
+########################################################################
+function innodb_checkpoint_lsn()
+{
+    ${MYSQL} ${MYSQL_ARGS} -e "SHOW ENGINE InnoDB STATUS\G" | \
+        grep "Last checkpoint at" | awk '{ print $4 }'
+}
+
+#check row count in a database/table
+#@in database
+#@in table
+#@in expected row count
+########################################################################
+function check_count() {
+  local db=$1
+  local table=$2
+  local expected_row_count=$3
+  local row_count=`$MYSQL $MYSQL_ARGS -Ns -e "select count(1) from $table" $db | awk {'print $1'}`
+  vlog "row count in table $table is $row_count"
+
+  if [ "$row_count" != $expected_row_count ]; then
+   vlog "rows in table t1 is $row_count when it should be $expected_row_count"
+   exit -1
+  fi
+}
+
+#general method to restore
+########################################################################
+function backup_and_restore() {
+  local db=$1
+  record_db_state $db
+  rm -rf $topdir/backup
+  mkdir -p $topdir/backup
+
+  ## backup and prepare based on if keyring is used
+  if [ -z ${plugin_dir+x} ]
+  then
+    xtrabackup --backup --target-dir=$topdir/backup
+    run_cmd $XB_BIN $XB_ARGS --prepare --target-dir=$topdir/backup
+  else
+    xtrabackup --backup --target-dir=$topdir/backup --xtrabackup-plugin-dir=${plugin_dir}
+    run_cmd $XB_BIN $XB_ARGS --xtrabackup-plugin-dir=${plugin_dir} --prepare \
+      --target-dir=$topdir/backup ${keyring_args}
+  fi
+
+  # Restore
+  stop_server
+  rm -rf $mysql_datadir
+  xtrabackup --copy-back --target-dir=$topdir/backup
+  start_server
+  # Verify backup
+  run_cmd verify_db_state $db
+}
+
+# Grep mysql.general_log for statements executed by xtrabackup
+#######################################################################
+function grep_general_log()
+{
+    ${MYSQL} ${MYSQL_ARGS} -Ne \
+        "SELECT argument FROM mysql.general_log WHERE command_type = 'Query' \
+            AND argument IN ('FLUSH NO_WRITE_TO_BINLOG TABLES', \
+                             'FLUSH TABLES WITH READ LOCK', \
+                             'FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS', \
+                             'UNLOCK TABLES', \
+                             'LOCK TABLES FOR BACKUP', \
+                             'LOCK INSTANCE FOR BACKUP', \
+                             'UNLOCK INSTANCE')"
+}
+
+### kill all running queries pattern is passed as argument or WHERE condition
+### example kill_condition_queries "time > 0 & INFO like '%SLEEP%'"
+function kill_query_pattern()
+{
+  local condition=$1
+  run_cmd $MYSQL $MYSQL_ARGS --force --batch  test <<EOF
+  select concat('KILL ',id,';') from information_schema.processlist
+  where $condition into outfile '$MYSQLD_TMPDIR/killcondition.sql';
+  source $MYSQLD_TMPDIR/killcondition.sql;
+EOF
+  rm -f $MYSQLD_TMPDIR/killcondition.sql
+}
+
+### wait for file to be generated()
+### It is used to ensure backup has been started by checking if xtrabackup_logfile
+## is generated
+function wait_for_file_to_generated() {
+  local file=$1
+  vlog "Wait for $file to be generated"
+  while [ ! -r "$file" ]
+  do
+    sleep 1
+    i=$((i+1))
+    echo "Waited $i seconds for $file to be created"
+  done
+}
+
 # To avoid unbound variable error when no server have been started
 SRV_MYSQLD_IDS=
