@@ -5,6 +5,7 @@
 #                       changed page bitmap output.
 #    ib_inc_extra_args: extra args to be passed to innobackup incremental 
 #                       backup invocations.
+#    ib_full_backup_extra_args: extra args to be passed to xtrabackup backup invocations.
 #    ib_inc_use_lsn:    if 1, use --incremental-lsn instead of
 #                       --incremental-basedir
 
@@ -14,8 +15,15 @@ MYSQLD_EXTRA_MY_CNF_OPTS="${MYSQLD_EXTRA_MY_CNF_OPTS:-""}
 innodb_file_per_table"
 
 ib_inc_use_lsn=${ib_inc_use_lsn:-""}
+ib_full_backup_extra_args=${ib_full_backup_extra_args:-""}
 
 start_server
+
+
+if [[ $ib_full_backup_extra_args =~ 'page-tracking' ]]; then
+  run_cmd ${MYSQL} ${MYSQL_ARGS} -e "INSTALL COMPONENT \"file://component_mysqlbackup\""
+fi
+
 load_dbase_schema incremental_sample
 
 # Adding initial rows
@@ -23,12 +31,12 @@ multi_row_insert incremental_sample.test \({1..100},100\)
 
 vlog "Starting backup"
 full_backup_dir=$topdir/full_backup
-innobackupex --no-timestamp $full_backup_dir
+xtrabackup --backup $ib_full_backup_extra_args  --target-dir=$full_backup_dir
 
 # Changing data
 
 vlog "Making changes to database"
-${MYSQL} ${MYSQL_ARGS} -e "create table t2 (a int(11) default null, number int(11) default null) engine=innodb" incremental_sample
+${MYSQL} ${MYSQL_ARGS} -e "create table t2 (a int(11) default null, number text default null) engine=innodb" incremental_sample
 
 multi_row_insert incremental_sample.test \({101..1000},1000\)
 multi_row_insert incremental_sample.t2 \({101..1000},1000\)
@@ -53,7 +61,7 @@ vlog "###############"
 
 # Incremental backup
 inc_backup_dir=$topdir/backup_incremental
-if [ $ib_inc_use_lsn = "1" ]
+if [ "$ib_inc_use_lsn" = "1" ]
 then
     inc_lsn=`grep to_lsn $full_backup_dir/xtrabackup_checkpoints | \
              sed 's/to_lsn = //'`
@@ -65,25 +73,32 @@ else
     ib_inc_extra_args="${ib_inc_extra_args:-""} --incremental-basedir=$full_backup_dir"
 fi
 
-innobackupex --no-timestamp --incremental $ib_inc_extra_args $inc_backup_dir
+$MYSQL $MYSQL_ARGS \
+    -e 'START TRANSACTION; DELETE FROM t2 LIMIT 1000; SELECT SLEEP(200000);' \
+    incremental_sample &
+job_id=$!
+
+xtrabackup --backup $ib_inc_extra_args --target-dir=$inc_backup_dir
+
+kill -SIGKILL $job_id
 
 vlog "Preparing backup"
 # Prepare backup
 vlog "##############"
 vlog "# PREPARE #1 #"
 vlog "##############"
-innobackupex --apply-log --redo-only $full_backup_dir
+xtrabackup --prepare --apply-log-only --target-dir=$full_backup_dir
 vlog "Log applied to full backup"
 vlog "##############"
 vlog "# PREPARE #2 #"
 vlog "##############"
-innobackupex --apply-log --redo-only --incremental-dir=$inc_backup_dir \
-    $full_backup_dir
+xtrabackup --prepare --apply-log-only --incremental-dir=$inc_backup_dir \
+    --target-dir=$full_backup_dir
 vlog "Delta applied to full backup"
 vlog "##############"
 vlog "# PREPARE #3 #"
 vlog "##############"
-innobackupex --apply-log $full_backup_dir
+xtrabackup --prepare --target-dir=$full_backup_dir
 vlog "Data prepared for restore"
 
 # Destroying mysql data
@@ -96,7 +111,7 @@ vlog "Copying files"
 vlog "###########"
 vlog "# RESTORE #"
 vlog "###########"
-innobackupex --copy-back $full_backup_dir
+xtrabackup --copy-back --target-dir=$full_backup_dir
 vlog "Data restored"
 
 start_server
@@ -118,3 +133,8 @@ then
 fi
 
 vlog "Checksums are OK"
+
+if egrep -q "We scanned the log up to [0-9]+. A checkpoint was at [0-9]+" $OUTFILE
+then
+    die "PXB-1699 is present"
+fi
