@@ -15,7 +15,7 @@ trap terminate SIGHUP SIGINT SIGQUIT SIGTERM
 result=0
 
 # Default test timeout in seconds
-TEST_TIMEOUT=900
+TEST_TIMEOUT=1800
 
 # Magic exit code to indicate a skipped test
 export SKIPPED_EXIT_CODE=200
@@ -42,15 +42,18 @@ Usage: $0 [-f] [-g] [-h] [-s suite] [-t test_name] [-d mysql_basedir] [-c build_
 -f          Continue running tests after failures
 -d path     Server installation directory. Default is './server'.
 -g          Debug mode
--t path     Run only a single named test
+-k          Make a copy of failed var directory
+-t path     Run only a single named test. This option can be passed multiple times.
 -h          Print this help message
--s suite    Select a test suite to run. Possible values: experimental, t. 
-            Default is 't'.
+-s suite    Select a test suite to run. Possible values: binlog, bitmap, experimental, gr, keyring, rocksdb, pagetracking,  main.
+            Default is 'binlog, main, gr, pagetracking, rocksdb and keyring'.
 -j N        Run tests in N parallel processes.
 -T seconds  Test timeout (default is $TEST_TIMEOUT seconds).
 -x options  Extra options to pass to xtrabackup
--i options  Extra options to pass to innobackupex
+-X options  Extra options to pass to mysqld
 -r path     Use specified path as root directory for test workers.
+-R          Run xtrabackup and mysqld under Record and Replay
+-D          Use mysqld-debug for debug test run.
 EOF
 }
 
@@ -92,6 +95,7 @@ function kill_worker()
 {
     local worker=$1
     local pid=${worker_pids[$worker]}
+    local cpids=`pgrep -P $pid`
 
     worker_pids[$worker]=""
 
@@ -103,6 +107,7 @@ function kill_worker()
 
     # First send SIGTERM to let worker exit gracefully
     kill -SIGTERM $pid >/dev/null 2>&1 || true
+    for cpid in $cpids ; do kill -SIGTERM $cpid >/dev/null 2>&1 ; done
 
     sleep 1
 
@@ -114,6 +119,7 @@ function kill_worker()
 
     # Now kill with SIGKILL
     kill -SIGKILL $pid >/dev/null 2>&1 || true
+    for cpid in $cpids ; do kill -SIGKILL $cpid >/dev/null 2>&1 ; done
     wait $pid >/dev/null 2>&1 || true
     release_port_locks $pid
 }
@@ -262,17 +268,29 @@ function set_vars()
 {
     if gnutar --version > /dev/null 2>&1
     then
-	TAR=gnutar
+        TAR=gnutar
     elif gtar --version > /dev/null 2>&1
     then
-	TAR=gtar
+        TAR=gtar
     else
-	TAR=tar
+        TAR=tar
     fi
 
-    find_program MYSQL_INSTALL_DB mysql_install_db $MYSQL_BASEDIR/bin \
-	$MYSQL_BASEDIR/scripts
-    find_program MYSQLD mysqld $MYSQL_BASEDIR/bin/ $MYSQL_BASEDIR/libexec
+    if gnused --version > /dev/null 2>&1
+    then
+        SED=gnused
+    elif gsed --version > /dev/null 2>&1
+    then
+        SED=gsed
+    else
+        SED=sed
+    fi
+
+    if [ "$MYSQL_DEBUG_MODE" = "on" ]; then
+        find_program MYSQLD mysqld-debug $MYSQL_BASEDIR/bin/ $MYSQL_BASEDIR/libexec
+    else
+        find_program MYSQLD mysqld $MYSQL_BASEDIR/bin/ $MYSQL_BASEDIR/libexec
+    fi
     find_program MYSQL mysql $MYSQL_BASEDIR/bin
     find_program MYSQLADMIN mysqladmin $MYSQL_BASEDIR/bin
     find_program MYSQLDUMP mysqldump $MYSQL_BASEDIR/bin
@@ -281,7 +299,8 @@ function set_vars()
     # appropriately
     if test -d $PWD/../src
     then
-        PATH="$PWD/..:$PWD/../src:$PATH"
+	PATH="$PWD/../../../../runtime_output_directory:$PWD/..:$PWD/../src/xbcloud:$PATH"
+        XTRABACKUP_BASEDIR="$PWD/../src"
     fi
 
     if test -d $PWD/../bin
@@ -289,6 +308,12 @@ function set_vars()
         PATH="$PWD/../bin:$PATH"
     fi
 
+    # 8.0 branch run test at xtrabackup-test/test folder
+    if test -d $PWD/../../bin
+    then
+        PATH="$PWD/../../bin:$PATH"
+    fi
+
     PATH="${MYSQL_BASEDIR}/bin:$PATH"
 
     if [ -z "${LD_LIBRARY_PATH:-}" ]; then
@@ -298,13 +323,68 @@ function set_vars()
     fi
     DYLD_LIBRARY_PATH="$LD_LIBRARY_PATH"
 
-    export TAR MYSQL_BASEDIR MYSQL MYSQLD MYSQLADMIN \
-MYSQL_INSTALL_DB PATH LD_LIBRARY_PATH DYLD_LIBRARY_PATH MYSQLDUMP
+    export TAR SED MYSQL_BASEDIR MYSQL MYSQLD MYSQLADMIN \
+MYSQL_INSTALL_DB PATH LD_LIBRARY_PATH DYLD_LIBRARY_PATH MYSQLDUMP \
+XTRABACKUP_BASEDIR
 
     # Use stage-v.percona.com as a VersionCheck server
     export PERCONA_VERSION_CHECK_URL=https://stage-v.percona.com
 }
 
+########################################################################
+# Configura ASAN suppress list
+########################################################################
+function config_asan()
+{
+  IS_ASAN=$(ldd $XB_BIN | grep asan | wc -l)
+  if [[ $IS_ASAN != 0 ]];
+  then
+    LSAN_OPTIONS=suppressions=${PWD}/asan.supp
+    export LSAN_OPTIONS
+    echo "This is ASAN build"
+  fi
+}
+
+
+# Configure mysql to read local component files if created by test cases.
+function config_local_components()
+{
+  local FILE="${MYSQLD}.my"
+  if [[ ! -f "${FILE}" ]]; then
+    cat <<EOF > "${FILE}"
+{
+  "read_local_manifest": true
+}
+EOF
+  fi
+
+  FILE="$MYSQL_BASEDIR/lib/plugin/component_keyring_file.cnf"
+  if [[ ! -f "${FILE}" ]]; then
+    cat <<EOF > "${FILE}"
+{
+  "read_local_config": true
+}
+EOF
+  fi
+
+  FILE="$MYSQL_BASEDIR/lib/plugin/component_keyring_kmip.cnf"
+  if [[ ! -f "${FILE}" ]]; then
+    cat <<EOF > "${FILE}"
+{
+ "read_local_config": true
+}
+EOF
+  fi
+
+  FILE="$MYSQL_BASEDIR/lib/plugin/component_keyring_kms.cnf"
+  if [[ ! -f "${FILE}" ]]; then
+    cat <<EOF > "${FILE}"
+{
+  "read_local_config": true
+}
+EOF
+  fi
+}
 
 # Fix innodb51 test failures on Centos5-32 Jenkins slaves due to SELinux
 # preventing shared symbol relocations in ha_innodb_plugin.so.0.0.0
@@ -325,7 +405,6 @@ function get_version_info()
     MYSQLD_EXTRA_ARGS=
 
     XB_BIN=""
-    IB_ARGS="--user=root --ibbackup=$XB_BIN ${IB_EXTRA_OPTS:-}"
     XB_ARGS="--no-defaults"
     XB_BIN="xtrabackup"
 
@@ -404,11 +483,7 @@ function get_version_info()
                 INNODB_VERSION="$MYSQL_VERSION"
             fi
             ;;
-        5.2 | 5.3 )
-            ;;
-        5.5 )
-            ;;
-        5.6 | 10.0 | 10.1)
+        8.0 )
             ;;
         *)
             vlog "Unknown MySQL/InnoDB version: $MYSQL_VERSION/$INNODB_VERSION"
@@ -424,20 +499,35 @@ function get_version_info()
     fi
     XB_BIN="$XB_PATH"
 
-    # Set the correct binary for innobackupex
-    IB_BIN="`which innobackupex`"
-    if [ -z "$IB_BIN" ]
-    then
-	vlog "Cannot find 'innobackupex' in PATH"
-	return 1
-    fi
-
     stop_server >>$OUTFILE 2>&1
 
     export MYSQL_VERSION MYSQL_VERSION_COMMENT MYSQL_FLAVOR \
 	INNODB_VERSION XTRADB_VERSION INNODB_FLAVOR \
-	XB_BIN IB_BIN IB_ARGS XB_ARGS MYSQLD_EXTRA_ARGS \
-        WSREP_READY LIBGALERA_PATH
+	XB_BIN XB_ARGS MYSQLD_EXTRA_ARGS WSREP_READY LIBGALERA_PATH
+}
+
+############################################################################
+# Make a copy of var directory
+############################################################################
+function copy_worker_var_dir()
+{
+  local keep_worker_number=$1
+  [ -d $TEST_BASEDIR/results/debug/ ] || mkdir $TEST_BASEDIR/results/debug/
+  for worker_dir in $TEST_BASEDIR/var/w+([0-9])
+  do
+      [ -d $worker_dir ] || continue
+      [[ $worker_dir =~ w([0-9]+)$ ]] || continue
+
+      local worker=${BASH_REMATCH[1]}
+
+      if [ "$worker" = "$keep_worker_number" ]
+      then
+          local tname=`basename $tpath .sh`
+          local debug_dir=$(mktemp -d -p $TEST_BASEDIR/results/debug/ -t ${tname}.XXXXXX)
+          cp -R ${worker_outfiles[$worker]} ${debug_dir}
+          cp -R $TEST_BASEDIR/var/w${worker} ${debug_dir}
+      fi
+  done
 }
 
 ###########################################################################
@@ -549,6 +639,15 @@ function reap_worker()
     local skip_file=${worker_skip_files[$worker]}
     local tpath=${worker_names[$worker]}
     local tname=`basename $tpath .sh`
+    local suite=`dirname $tpath | awk -F'/' '{print $NF}'`
+    if [[ "${suite}" = "t" ]];
+    then
+      suite="main"
+    fi
+    if [[ $NOFSUITES -ge 1 ]];
+    then
+      tname="${suite}.${tname}"
+    fi
     local test_time=$((`now` - worker_stime[$worker]))
     local status_file=${worker_status_files[$worker]}
     local rc
@@ -610,6 +709,10 @@ function reap_worker()
 
            if [ -z "$DEBUG" ]
            then
+                if [ ! -z "$KEEP_VAR" ]
+                then
+                  copy_worker_var_dir $worker
+                fi
                cleanup_worker $worker
            else
                DEBUG_WORKER=$worker
@@ -617,6 +720,10 @@ function reap_worker()
 
            return 1
        else
+           if [ ! -z "$KEEP_VAR" ]
+           then
+             copy_worker_var_dir $worker
+           fi
            cleanup_worker $worker
 
            return 0
@@ -747,29 +854,43 @@ function cleanup_on_test_exit()
 TEST_START_TIME=`now`
 
 tname=""
+suites=""
+NOFSUITES=0
 XTRACE_OPTION=""
 force=""
+KEEP_VAR=""
 SUBUNIT_OUT=test_results.subunit
 NWORKERS=
 DEBUG_WORKER=""
+MYSQL_DEBUG_MODE=off
 
-while getopts "fgh?:t:s:d:c:j:T:x:i:r:" options; do
+while getopts "fgkhDR?:t:s:d:c:j:T:x:X:i:r:" options; do
         case $options in
             f ) force="yes";;
             t )
 
-                tname="$OPTARG";
-                if ! [ -r "$tname" ]
+                tname="${tname} $OPTARG";
+                if ! [ -r "$OPTARG" ]
                 then
-                    echo "Cannot find test $tname."
+                    echo "Cannot find test $OPTARG."
                     exit -1
                 fi
                 ;;
 
             g ) DEBUG=on;;
+            k ) KEEP_VAR=on;;
             h ) usage; exit;;
-            s ) tname="$OPTARG/*.sh";;
+            s )
+                if [[ "$OPTARG" = "default" ]];
+                then
+                  tname="${tname} t/*.sh";
+                else
+                  tname="${tname} suites/$OPTARG/*.sh";
+                fi
+                suites="${suites} $OPTARG"
+                ;;
             d ) export MYSQL_BASEDIR="$OPTARG";;
+            D ) export MYSQL_DEBUG_MODE=on ;;
             c ) echo "Warning: -c does not have any effect and is only \
 recognized for compatibility";;
             j )
@@ -795,8 +916,8 @@ recognized for compatibility";;
                 XB_EXTRA_MY_CNF_OPTS="$OPTARG"
                 ;;
 
-            i )
-                IB_EXTRA_OPTS="$OPTARG"
+            X )
+                MYSQLD_EXTRA_MY_CNF_OPTS="$OPTARG"
                 ;;
 
             r )
@@ -809,6 +930,7 @@ recognized for compatibility";;
                 TEST_VAR_ROOT="$TEST_BASEDIR/var"
                 ;;
 
+            R ) export WITH_RR=1 ;;
             ? ) echo "Use \`$0 -h' for the list of available options."
                     exit -1;;
         esac
@@ -820,7 +942,8 @@ if [ -n "$tname" ]
 then
    tests="$tname"
 else
-   tests="t/*.sh"
+   tests="suites/binlog/* suites/pagetracking/* suites/gr/*.sh suites/keyring/*.sh suites/rocksdb/*.sh t/*.sh"
+   suites=" pagetracking binlog gr keyring rocksdb main"
 fi
 
 export OUTFILE="$PWD/results/setup"
@@ -842,6 +965,20 @@ then
     exit -1
 fi
 
+config_asan
+
+if is_server_version_higher_than 8.0.23
+then
+  config_local_components
+fi
+
+if test -d $PWD/../../../../plugin_output_directory
+then
+  plugin_dir=$PWD/../../../../plugin_output_directory
+else
+  plugin_dir=$PWD/../../lib/plugin/
+fi
+
 echo "Running against $MYSQL_FLAVOR $MYSQL_VERSION ($INNODB_FLAVOR $INNODB_VERSION)" |
   tee -a $OUTFILE
 
@@ -853,6 +990,12 @@ if [ "$NWORKERS" -gt 1 ]
 then
     echo "Using $NWORKERS parallel workers" | tee -a $OUTFILE
 fi
+
+if [ -n  "$suites" ];
+then
+  echo "Running suite(s):${suites}"
+  NOFSUITES=$(echo ${suites}| tr -cd ' \t' | wc -c)
+fi
 echo | tee -a $OUTFILE
 
 cat <<EOF
@@ -901,6 +1044,15 @@ do
    TOTAL_COUNT=$((TOTAL_COUNT+1))
 
    name=`basename $t .sh`
+   suite=`dirname $t | awk -F'/' '{print $NF}'`
+   if [[ "${suite}" = "t" ]];
+   then
+     suite="main"
+   fi
+   if [[ $NOFSUITES -ge 1 ]];
+   then
+     name="${suite}.${name}"
+   fi
    worker_names[$worker]=$t
    worker_outfiles[$worker]="$PWD/results/$name"
    worker_skip_files[$worker]="$PWD/results/$name.skipped"
