@@ -15,7 +15,7 @@ slave_id=2
 start_server_with_id $master_id
 start_server_with_id $slave_id
 
-setup_slave $slave_id $master_id
+setup_slave GTID $slave_id $master_id
 
 switch_server $master_id
 
@@ -39,7 +39,7 @@ switch_server $slave_id
 
 $MYSQL $MYSQL_ARGS -e "SHOW SLAVE STATUS\G"
 
-innobackupex --no-timestamp --slave-info $topdir/backup
+xtrabackup --backup --slave-info --target-dir=$topdir/backup
 
 vlog "Verifying format of xtrabackup_slave_info:"
 f=$topdir/backup/xtrabackup_slave_info
