@@ -17,16 +17,16 @@ test "`$MYSQL $MYSQL_ARGS -Ns -e "SELECT @@innodb_fast_checksum"`" = "1"
 load_sakila
 
 backup_dir=${topdir}/backup
-innobackupex --no-timestamp $backup_dir
+xtrabackup --backup --target-dir=$backup_dir
 
 record_db_state sakila
 
 stop_server
 rm -rf $MYSQLD_DATADIR/*
 
-innobackupex --apply-log $backup_dir
+xtrabackup --prepare --target-dir=$backup_dir
 
-innobackupex --copy-back $backup_dir
+xtrabackup --copy-back --target-dir=$backup_dir
 
 start_server
 
