@@ -1,15 +1,21 @@
 /*
-   Copyright 2010 Sun Microsystems, Inc.
-   All rights reserved. Use is subject to license terms.
+   Copyright (c) 2010, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -52,12 +58,12 @@ import testsuite.clusterj.model.CharsetUtf8;
  *   d) create a String from the characters in the CharBuffer that could fit into the column
  *   e) add the String to the list of String
  *   f) continue from c) until all characters have been represented in the list of String
- *   g) remove all rows of the table
- *   h) use JDBC or clusterj to write a row in the database for each String in the list
- *   i) use JDBC or clusterj to read all rows and compare the String to the list of Strings
+ *   g) use JDBC or clusterj to write a row in the database for each String in the list
+ *   h) use JDBC or clusterj to read all rows and compare the String to the list of Strings
+ *   i) remove all rows of the table
  *
  */
-public class CharsetTest extends AbstractClusterJModelTest {
+public class CharsetTest extends AbstractClusterJTest {
 
     @Override
     public void localSetUp() {
@@ -66,147 +72,99 @@ public class CharsetTest extends AbstractClusterJModelTest {
         setAutoCommit(connection, false);
     }
 
+    @Override
+    public Properties modifyProperties() {
+        // Modify JDBC properties to add server character encoding
+        Properties modifiedProps = new Properties();
+        modifiedProps.putAll(props);
+        modifiedProps.put("characterEncoding", "utf8");
+        return modifiedProps;
+    }
+
     public void testLatin1() {
-        writeJDBCreadJDBC("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.SMALL);
-        writeJDBCreadJDBC("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.MEDIUM);
-        writeJDBCreadJDBC("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.LARGE);
-
-        writeJDBCreadNDB("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.SMALL);
-        writeJDBCreadNDB("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.MEDIUM);
-        writeJDBCreadNDB("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.LARGE);
-
-        writeNDBreadJDBC("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.SMALL);
-        writeNDBreadJDBC("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadJDBC("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.LARGE);
-
-        writeNDBreadNDB("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.SMALL);
-        writeNDBreadNDB("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadNDB("windows-1252", "charsetlatin1", CharsetLatin1.class, ColumnDescriptor.LARGE);
-
-        failOnError();
+        testCharset("windows-1252", "charsetlatin1", CharsetLatin1.class);
     }
 
     public void testUtf8() {
-        writeJDBCreadJDBC("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.SMALL);
-        writeJDBCreadJDBC("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.MEDIUM);
-        writeJDBCreadJDBC("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.LARGE);
-
-        writeJDBCreadNDB("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.SMALL);
-        writeJDBCreadNDB("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.MEDIUM);
-        writeJDBCreadNDB("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.LARGE);
-
-        writeNDBreadJDBC("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.SMALL);
-        writeNDBreadJDBC("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadJDBC("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.LARGE);
-
-        writeNDBreadNDB("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.SMALL);
-        writeNDBreadNDB("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadNDB("UTF-8", "charsetutf8", CharsetUtf8.class, ColumnDescriptor.LARGE);
-
-        failOnError();
+        testCharset("UTF-8", "charsetutf8", CharsetUtf8.class);
     }
 
     public void testSjis() {
-        /* These tests are excluded due to a JDBC error:
-         * java.sql.SQLException: 
-         * Failed to insert charsetsjis at instance 0 errant string: [... 165 167 168... ]
-         * Incorrect string value: '\xC2\xA5\xC2\xA7\xC2\xA8...' for column 'smallcolumn' at row 1
-                                at com.mysql.jdbc.SQLError.createSQLException(SQLError.java:1055)
-                                at com.mysql.jdbc.SQLError.createSQLException(SQLError.java:956)
-                                at com.mysql.jdbc.MysqlIO.checkErrorPacket(MysqlIO.java:3558)
-                                at com.mysql.jdbc.MysqlIO.checkErrorPacket(MysqlIO.java:3490)
-                                at com.mysql.jdbc.MysqlIO.sendCommand(MysqlIO.java:1959)
-                                at com.mysql.jdbc.MysqlIO.sqlQueryDirect(MysqlIO.java:2109)
-                                at com.mysql.jdbc.ConnectionImpl.execSQL(ConnectionImpl.java:2648)
-                                at com.mysql.jdbc.PreparedStatement.executeInternal(PreparedStatement.java:2077)
-                                at com.mysql.jdbc.PreparedStatement.execute(PreparedStatement.java:1356)
-                                at testsuite.clusterj.CharsetTest.writeToJDBC(CharsetTest.java:317)
-        writeJDBCreadJDBC("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.SMALL);
-        writeJDBCreadJDBC("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.MEDIUM);
-        writeJDBCreadJDBC("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.LARGE);
-         */
-
-        writeNDBreadJDBC("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.SMALL);
-        writeNDBreadJDBC("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadJDBC("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.LARGE);
-
-        writeNDBreadNDB("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.SMALL);
-        writeNDBreadNDB("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadNDB("SJIS", "charsetsjis", CharsetSjis.class, ColumnDescriptor.LARGE);
-
-        failOnError();
+        testCharset("SJIS", "charsetsjis", CharsetSjis.class);
     }
 
     public void testBig5() {
-        writeJDBCreadJDBC("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.SMALL);
-        writeJDBCreadJDBC("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.MEDIUM);
-        writeJDBCreadJDBC("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.LARGE);
+        testCharset("big5", "charsetbig5", CharsetBig5.class);
+    }
 
-        writeJDBCreadNDB("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.SMALL);
-        writeJDBCreadNDB("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.MEDIUM);
-        writeJDBCreadNDB("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.LARGE);
+    protected void testCharset(String charsetName, String tableName,
+            Class<? extends CharsetModel> modelClass) {
 
-        writeNDBreadJDBC("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.SMALL);
-        writeNDBreadJDBC("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadJDBC("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.LARGE);
+        writeJDBCreadJDBC(charsetName, tableName, modelClass, ColumnDescriptor.SMALL);
+        writeJDBCreadJDBC(charsetName, tableName, modelClass, ColumnDescriptor.MEDIUM);
+        writeJDBCreadJDBC(charsetName, tableName, modelClass, ColumnDescriptor.LARGE);
 
-        writeNDBreadNDB("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.SMALL);
-        writeNDBreadNDB("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.MEDIUM);
-        writeNDBreadNDB("big5", "charsetbig5", CharsetBig5.class, ColumnDescriptor.LARGE);
+        writeJDBCreadNDB(charsetName, tableName, modelClass, ColumnDescriptor.SMALL);
+        writeJDBCreadNDB(charsetName, tableName, modelClass, ColumnDescriptor.MEDIUM);
+        writeJDBCreadNDB(charsetName, tableName, modelClass, ColumnDescriptor.LARGE);
+
+        writeNDBreadJDBC(charsetName, tableName, modelClass, ColumnDescriptor.SMALL);
+        writeNDBreadJDBC(charsetName, tableName, modelClass, ColumnDescriptor.MEDIUM);
+        writeNDBreadJDBC(charsetName, tableName, modelClass, ColumnDescriptor.LARGE);
+
+        writeNDBreadNDB(charsetName, tableName, modelClass, ColumnDescriptor.SMALL);
+        writeNDBreadNDB(charsetName, tableName, modelClass, ColumnDescriptor.MEDIUM);
+        writeNDBreadNDB(charsetName, tableName, modelClass, ColumnDescriptor.LARGE);
 
         failOnError();
     }
 
     protected void writeJDBCreadJDBC(String charsetName, String tableName, Class<? extends CharsetModel> modelClass,
             ColumnDescriptor columnDescriptor) {
-        removeAll(modelClass);
         List<String> result = null;
         List<String> strings = generateStrings(columnDescriptor, charsetName);
         List<CharsetModel> instances = generateInstances(columnDescriptor, modelClass, strings);
         writeToJDBC(columnDescriptor, tableName, instances);
         result = readFromJDBC(columnDescriptor, tableName);
         if (debug) System.out.println("Returned results of size " + result.size());
-//        if (debug) System.out.println("Results:\n" + dump(result));
-        verify("writeJDBCreadJDBC", strings, result, columnDescriptor);
+        verify("writeJDBCreadJDBC ", strings, result, columnDescriptor);
+        removeAll(modelClass);
     }
 
     protected void writeJDBCreadNDB(String charsetName, String tableName, Class<? extends CharsetModel> modelClass,
             ColumnDescriptor columnDescriptor) {
-        removeAll(modelClass);
         List<String> result = null;
         List<String> strings = generateStrings(columnDescriptor, charsetName);
         List<CharsetModel> instances = generateInstances(columnDescriptor, modelClass, strings);
         writeToJDBC(columnDescriptor, tableName, instances);
         result = readFromNDB(columnDescriptor, modelClass);
         if (debug) System.out.println("Returned results of size " + result.size());
-//        if (debug) System.out.println("Results: " + dump(result));
-        verify("writeJDBCreadNDB", strings, result, columnDescriptor);
+        verify("writeJDBCreadNDB ", strings, result, columnDescriptor);
+        removeAll(modelClass);
     }
 
     protected void writeNDBreadJDBC(String charsetName, String tableName, Class<? extends CharsetModel> modelClass,
             ColumnDescriptor columnDescriptor) {
-        removeAll(modelClass);
         List<String> result = null;
         List<String> strings = generateStrings(columnDescriptor, charsetName);
         List<CharsetModel> instances = generateInstances(columnDescriptor, modelClass, strings);
         writeToNDB(columnDescriptor, instances);
         result = readFromJDBC(columnDescriptor, tableName);
         if (debug) System.out.println("Returned results of size " + result.size());
-//        if (debug) System.out.println("Results: " + dump(result));
-        verify("writeNDBreadJDBC", strings, result, columnDescriptor);
+        verify("writeNDBreadJDBC ", strings, result, columnDescriptor);
+        removeAll(modelClass);
     }
 
     protected void writeNDBreadNDB(String charsetName, String tableName, Class<? extends CharsetModel> modelClass,
             ColumnDescriptor columnDescriptor) {
-        removeAll(modelClass);
         List<String> result = null;
         List<String> strings = generateStrings(columnDescriptor, charsetName);
         List<CharsetModel> instances = generateInstances(columnDescriptor, modelClass, strings);
         writeToNDB(columnDescriptor, instances);
         result = readFromNDB(columnDescriptor, modelClass);
         if (debug) System.out.println("Returned results of size " + result.size());
-//        if (debug) System.out.println("Results: " + dump(result));
-        verify("writeNDBreadNDB", strings, result, columnDescriptor);
+        verify("writeNDBreadNDB ", strings, result, columnDescriptor);
+        removeAll(modelClass);
     }
 
     private void verify(String where, List<String> expecteds, List<String> actuals, ColumnDescriptor columnDescriptor) {
@@ -214,15 +172,19 @@ public class CharsetTest extends AbstractClusterJModelTest {
         for (int i = 0; i < expecteds.size(); ++i) {
             String expected = expecteds.get(i);
             String actual = actuals.get(i);
-            int expectedLength = expected.length();
-            int actualLength = actual.length();
-            errorIfNotEqual(where + " got failure on size of column data for column width " + columnDescriptor.columnWidth + " at row " + i, expectedLength, actualLength);
-            if (expectedLength != actualLength) 
-                continue;
-            for (int j = 0; j < expected.length(); ++j) {
-                if (--maxErrors > 0) {
-                    errorIfNotEqual("Failure to match column data for column width " + columnDescriptor.columnWidth + " at row " + i + " column " + j,
-                            expected.codePointAt(j), actual.codePointAt(j));
+            if (actual == null) {
+                error(where + columnDescriptor.columnName + " actual column " + i + " was null.");
+            } else {
+                int expectedLength = expected.length();
+                int actualLength = actual.length();
+                errorIfNotEqual(where + "got failure on size of column data for column width " + columnDescriptor.columnWidth + " at row " + i, expectedLength, actualLength);
+                if (expectedLength != actualLength) 
+                    continue;
+                for (int j = 0; j < expected.length(); ++j) {
+                    if (--maxErrors > 0) {
+                        errorIfNotEqual("Failure to match column data for column width " + columnDescriptor.columnWidth + " at row " + i + " column " + j,
+                                expected.codePointAt(j), actual.codePointAt(j));
+                    }
                 }
             }
         }
@@ -236,7 +198,6 @@ public class CharsetTest extends AbstractClusterJModelTest {
         CharsetEncoder encoder = charset.newEncoder();
        // add all encodable characters to the buffer
         int count = 0;
-//        for (int i = 0; i < 65536; ++i) {
         for (int i = 0; i < 65536; ++i) {
             Character ch = (char)i;
             if (encoder.canEncode(ch)) {
@@ -301,10 +262,6 @@ public class CharsetTest extends AbstractClusterJModelTest {
         int i = 0;
         String value = "";
         try {
-            Properties extraProperties = new Properties();
-            extraProperties.put("characterEncoding", "utf8");
-            getConnection(extraProperties);
-            setAutoCommit(connection, false);
             preparedStatement = connection.prepareStatement(statement);
             if (debug) System.out.println(preparedStatement.toString());
             for (i = 0; i < instances.size(); ++i) {
@@ -312,8 +269,6 @@ public class CharsetTest extends AbstractClusterJModelTest {
                 preparedStatement.setInt(1, instance.getId());
                 value = columnDescriptor.get(instance);
                 preparedStatement.setString(2, value);
-//                if (debug) System.out.println("Value set to column is size " + value.length());
-//                if (debug) System.out.println(" value " + value);
                 preparedStatement.execute();
             }
             connection.commit();
