@@ -1,15 +1,21 @@
 /*
-   Copyright 2010 Sun Microsystems, Inc.
-   All rights reserved. Use is subject to license terms.
+   Copyright (c) 2010, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -27,9 +33,12 @@ import com.mysql.clusterj.annotation.PrimaryKey;
 drop table if exists blobtypes;
 create table blobtypes (
  id int not null primary key,
+ id_null_none int,
+ id_null_hash int,
 
  blobbytes blob,
- blobstream blob,
+
+ unique key idx_id_null_hash (id_null_hash) using hash
 
 ) ENGINE=ndbcluster DEFAULT CHARSET=latin1;
 
@@ -44,12 +53,14 @@ public interface BlobTypes extends IdBase {
     int getId();
     void setId(int id);
 
+    int getId_null_none();
+    void setId_null_none(int id);
+
+    int getId_null_hash();
+    void setId_null_hash(int id);
+
     @Lob
     byte[] getBlobbytes();
     void setBlobbytes(byte[] value);
 
-//    @Lob
-//    InputStream getBlobstream();
-//    void setBlobstream(InputStream value);
-
 }
