@@ -1,15 +1,21 @@
 /*
-   Copyright (C) 2003-2006, 2008 MySQL AB
-    All rights reserved. Use is subject to license terms.
+   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -21,6 +27,9 @@
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 80
+
+
 class CreateFragmentationReq {
   /**
    * Sender(s)
@@ -35,10 +44,11 @@ class CreateFragmentationReq {
   friend bool printCREATE_FRAGMENTATION_REQ(FILE *, 
 					    const Uint32 *, Uint32, Uint16);
 public:
-  STATIC_CONST( SignalLength = 7 );
+  static constexpr Uint32 SignalLength = 8;
   
   enum RequestInfo {
-    RI_ADD_PARTITION = 0x1,
+    RI_CREATE_FRAGMENTATION = 0x0,
+    RI_ADD_FRAGMENTS = 0x1,
     RI_GET_FRAGMENTATION = 0x2
   };
 private:
@@ -49,6 +59,8 @@ private:
   Uint32 noOfFragments;
   Uint32 primaryTableId;  // use same fragmentation as this table if not RNIL
   Uint32 map_ptr_i;
+  Uint32 partitionBalance;
+  Uint32 partitionCount;
 };
 
 class CreateFragmentationRef {
@@ -65,7 +77,7 @@ class CreateFragmentationRef {
   friend bool printCREATE_FRAGMENTATION_REF(FILE *, 
 					    const Uint32 *, Uint32, Uint16);
 public:
-  STATIC_CONST( SignalLength = 3 );
+  static constexpr Uint32 SignalLength = 3;
  
   enum ErrorCode {
     OK = 0
@@ -94,7 +106,7 @@ class CreateFragmentationConf {
   friend bool printCREATE_FRAGMENTATION_CONF(FILE *, 
 					     const Uint32 *, Uint32, Uint16);
 public:
-  STATIC_CONST( SignalLength = 4 );
+  static constexpr Uint32 SignalLength = 4;
   SECTION( FRAGMENTS = 0 );
   
 private:
@@ -104,4 +116,7 @@ private:
   Uint32 noOfFragments;
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
