@@ -1,15 +1,21 @@
 /*
-   Copyright (C) 2003, 2005-2007 MySQL AB
-    All rights reserved. Use is subject to license terms.
+   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -19,6 +25,9 @@
 #ifndef START_PERM_REQ_HPP
 #define START_PERM_REQ_HPP
 
+#define JAM_FILE_ID 204
+
+
 /**
  * This signal is sent by starting DIH to master DIH
  *
@@ -32,7 +41,7 @@ class StartPermReq {
   friend class Dbdih;
   
 public:
-  STATIC_CONST( SignalLength = 3 );
+  static constexpr Uint32 SignalLength = 3;
 private:
   
   Uint32 blockRef;
@@ -47,7 +56,7 @@ class StartPermConf {
   friend class Dbdih;
   
 public:
-  STATIC_CONST( SignalLength = 3 );
+  static constexpr Uint32 SignalLength = 3;
 private:
   
   Uint32 startingNodeId;
@@ -62,7 +71,7 @@ class StartPermRef {
   friend class Dbdih;
   
 public:
-  STATIC_CONST( SignalLength = 2 );
+  static constexpr Uint32 SignalLength = 2;
 private:
   
   Uint32 startingNodeId;
@@ -75,4 +84,30 @@ private:
     InitialStartRequired = 320
   };
 };
+
+class StartPermRep
+{
+  /**
+   * Sender(s)
+   */
+  friend class Dbdih;
+  /**
+   * Sender(s) / Reciver(s)
+   */
+  friend class Ndbcntr;
+
+public:
+  static constexpr Uint32 SignalLength = 2;
+  enum
+  {
+    PermissionToStart = 0,
+    CompletedStart = 1
+  };
+
+private:
+  Uint32 startNodeId;
+  Uint32 reason;
+};
+#undef JAM_FILE_ID
+
 #endif
