@@ -1,17 +1,24 @@
-/* Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "Dbinfo.hpp"
 
@@ -27,16 +34,19 @@
 #include <debugger/DebuggerNames.hpp>
 #endif
 
+#define JAM_FILE_ID 455
+
+
 Uint32 dbinfo_blocks[] = { DBACC, DBTUP, BACKUP, DBTC, SUMA, DBUTIL,
                            TRIX, DBTUX, DBDICT, CMVMI, DBLQH, LGMAN,
-                           PGMAN, DBSPJ, 0};
+                           PGMAN, DBSPJ, THRMAN, TRPMAN, QMGR, DBDIH, 0};
 
 Dbinfo::Dbinfo(Block_context& ctx) :
   SimulatedBlock(DBINFO, ctx)
 {
   BLOCK_CONSTRUCTOR(Dbinfo);
 
-  STATIC_ASSERT(sizeof(DbinfoScanCursor) == sizeof(Ndbinfo::ScanCursor));
+  static_assert(sizeof(DbinfoScanCursor) == sizeof(Ndbinfo::ScanCursor));
 
   /* Add Received Signals */
   addRecSignal(GSN_STTOR, &Dbinfo::execSTTOR);
@@ -70,9 +80,44 @@ void Dbinfo::execREAD_CONFIG_REQ(Signal *signal)
   const ReadConfigReq * req = (ReadConfigReq*)signal->getDataPtr();
   Uint32 ref = req->senderRef;
   Uint32 senderData = req->senderData;
+  Uint32 ntable;
 
-  /* In the future, do something sensible here. */
+  const ndb_mgm_configuration_iterator * p =
+     m_ctx.m_config.getOwnConfigIterator();
+  ndbrequire(p != 0);
+  ndbrequire(!ndb_mgm_get_int_parameter(p, CFG_DB_NO_TABLES, &ntable));
 
+  // Estimated number of tables. Without actually counting the tables in dict,
+  // this estimate could be far off. Take the configured maximum and divide
+  // by 3; then any actual value in the range from 11% to 100% will be off by
+  // a factor of 3 at most.
+  counts.est_tables = ntable / 3;
+
+  // Count nodes
+  for(int i = 1 ; i < MAX_NODES ; i++) {
+    NodeInfo::NodeType type = getNodeInfo(i).getType();
+    if(type == NodeInfo::NodeType::DB)
+      counts.data_nodes++;
+    if(type == NodeInfo::NodeType::DB || type == NodeInfo::NodeType::API ||
+       type == NodeInfo::NodeType::MGM)
+      counts.all_nodes++;
+  }
+
+  // Count threads
+  const THRConfigApplier & thr_cf =
+    globalEmulatorData.theConfiguration->m_thr_config;
+  counts.threads.send = globalData.ndbMtSendThreads;
+  counts.threads.db = thr_cf.getThreadCount();
+  counts.threads.ldm = thr_cf.getThreadCount(THRConfig::T_LDM);
+  counts.cpus = Ndb_GetHWInfo(false)->cpu_cnt;
+
+  // Count block instances
+  counts.log_parts = globalData.ndbLogParts;
+  counts.instances.tc  = globalData.ndbMtTcWorkers;
+  counts.instances.lqh = globalData.ndbMtLqhWorkers;
+  counts.instances.pgman = counts.instances.lqh + 1;
+
+  // Send conf
   ReadConfigConf * conf = (ReadConfigConf*)signal->getDataPtrSend();
   conf->senderRef = reference();
   conf->senderData = senderData;
@@ -84,9 +129,8 @@ void Dbinfo::sendSTTORRY(Signal* signal)
 {
   signal->theData[0] = 0;
   signal->theData[3] = 1;
-  signal->theData[4] = 3;
-  signal->theData[5] = 255; // No more start phases from missra
-  sendSignal(NDBCNTR_REF, GSN_STTORRY, signal, 6, JBB);
+  signal->theData[4] = 255; // No more start phases from missra
+  sendSignal(NDBCNTR_REF, GSN_STTORRY, signal, 5, JBB);
 }
 
 void Dbinfo::execDUMP_STATE_ORD(Signal* signal)
@@ -97,27 +141,27 @@ void Dbinfo::execDUMP_STATE_ORD(Signal* signal)
   {
   case DumpStateOrd::DbinfoListTables:
     jam();
-    ndbout_c("--- BEGIN NDB$INFO.TABLES ---");
+    g_eventLogger->info("--- BEGIN NDB$INFO.TABLES ---");
     for(int i = 0; i < Ndbinfo::getNumTables(); i++)
     {
       const Ndbinfo::Table& tab = Ndbinfo::getTable(i);
-      ndbout_c("%d,%s", i, tab.m.name);
+      g_eventLogger->info("%d,%s", i, tab.m.name);
     }
-    ndbout_c("--- END NDB$INFO.TABLES ---");
+    g_eventLogger->info("--- END NDB$INFO.TABLES ---");
     break;
 
   case DumpStateOrd::DbinfoListColumns:
     jam();
-    ndbout_c("--- BEGIN NDB$INFO.COLUMNS ---");
+    g_eventLogger->info("--- BEGIN NDB$INFO.COLUMNS ---");
     for(int i = 0; i < Ndbinfo::getNumTables(); i++)
     {
       const Ndbinfo::Table& tab = Ndbinfo::getTable(i);
 
       for(int j = 0; j < tab.m.ncols; j++)
-        ndbout_c("%d,%d,%s,%d", i, j,
-                 tab.col[j].name, tab.col[j].coltype);
+        g_eventLogger->info("%d,%d,%s,%d", i, j, tab.col[j].name,
+                            tab.col[j].coltype);
     }
-    ndbout_c("--- END NDB$INFO.COLUMNS ---");
+    g_eventLogger->info("--- END NDB$INFO.COLUMNS ---");
     break;
 
   };
@@ -144,8 +188,8 @@ switchRef(Uint32 block, Uint32 node)
 {
   const Uint32 ref = numberToRef(block, node);
 #ifdef DBINFO_SCAN_TRACE
-  ndbout_c("Dbinfo: switching to %s in node %d, ref: 0x%.8x",
-           getBlockName(block, "<unknown>"), node, ref);
+  g_eventLogger->info("Dbinfo: switching to %s in node %d, ref: 0x%.8x",
+                      getBlockName(block, "<unknown>"), node, ref);
 #endif
   return ref;
 }
@@ -258,6 +302,7 @@ void Dbinfo::execDBINFO_SCANREQ(Signal *signal)
       row.write_uint32(tableId);
       row.write_string(tab.m.name);
       row.write_string(tab.m.comment);
+      row.write_uint32(tab.m.estimate_rows(counts));
       ndbinfo_send_row(signal, req, row, rl);
 
       tableId++;
@@ -379,8 +424,7 @@ void Dbinfo::execDBINFO_SCANCONF(Signal *signal)
   ndbrequire(conf.cursor_sz == Ndbinfo::ScanCursor::Length);
 
   // Validate tableId
-  const Uint32 tableId= conf.tableId;
-  ndbassert(tableId < (Uint32)Ndbinfo::getNumTables());
+  ndbassert(conf.tableId < (Uint32)Ndbinfo::getNumTables());
 
   const Uint32 resultRef = conf.resultRef;
 
@@ -448,6 +492,22 @@ void Dbinfo::execNODE_FAILREP(Signal* signal)
 
   NodeFailRep * rep = (NodeFailRep*)signal->getDataPtr();
 
+  if(signal->getLength() == NodeFailRep::SignalLength)
+  {
+    ndbrequire(signal->getNoOfSections() == 1);
+    ndbrequire(getNodeInfo(refToNode(signal->getSendersBlockRef())).m_version);
+    SegmentedSectionPtr ptr;
+    SectionHandle handle(this, signal);
+    ndbrequire(handle.getSection(ptr, 0));
+    memset(rep->theNodes, 0, sizeof(rep->theNodes));
+    copy(rep->theNodes, ptr);
+    releaseSections(handle);
+  }
+  else
+  {
+    memset(rep->theNodes + NdbNodeBitmask48::Size, 0,
+           _NDB_NBM_DIFF_BYTES);
+  }
   Uint32 theFailedNodes[NdbNodeBitmask::Size];
   for (Uint32 i = 0; i < NdbNodeBitmask::Size; i++)
     theFailedNodes[i] = rep->theNodes[i];
