@@ -1,15 +1,21 @@
 /*
-   Copyright (C) 2005-2007 MySQL AB, 2009 Sun Microsystems, Inc.
-    All rights reserved. Use is subject to license terms.
+   Copyright (c) 2005, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -20,43 +26,42 @@
 #include "Dbtux.hpp"
 #include <math.h>
 
+#define JAM_FILE_ID 367
+
+
 // debug note: uses new-style debug macro "D" unlike rest of DBTUX
 // there is no filtering feature (yet) like "DebugStat"
 
 void
-Dbtux::execREAD_PSEUDO_REQ(Signal* signal)
+Dbtux::execREAD_PSEUDO_REQ(Uint32 scanPtrI, Uint32 attrId, Uint32* out, Uint32 out_words)
 {
   jamEntry();
   ScanOpPtr scanPtr;
-  scanPtr.i = signal->theData[0];
-  c_scanOpPool.getPtr(scanPtr);
+  scanPtr.i = scanPtrI;
+  ndbrequire(c_scanOpPool.getValidPtr(scanPtr));
   StatOpPtr statPtr;
   statPtr.i = scanPtr.p->m_statOpPtrI;
 
-  Uint32 attrId = signal->theData[1];
-  Uint32* out = &signal->theData[0];
-
   switch (attrId) {
   case AttributeHeader::RECORDS_IN_RANGE:
     jam();
     ndbrequire(statPtr.i == RNIL);
-    statRecordsInRange(scanPtr, out);
+    statRecordsInRange(scanPtr, out, out_words);
     break;
   case AttributeHeader::INDEX_STAT_KEY:
     jam();
     ndbrequire(statPtr.i != RNIL);
     c_statOpPool.getPtr(statPtr);
-    statScanReadKey(statPtr, out);
+    statScanReadKey(statPtr, out, out_words);
     break;
   case AttributeHeader::INDEX_STAT_VALUE:
     jam();
     ndbrequire(statPtr.i != RNIL);
     c_statOpPool.getPtr(statPtr);
-    statScanReadValue(statPtr, out);
+    statScanReadValue(statPtr, out, out_words);
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 }
 
@@ -73,7 +78,7 @@ Dbtux::execREAD_PSEUDO_REQ(Signal* signal)
  * 3) after range.  1-3) are estimates and need not add up to 0).
  */
 void
-Dbtux::statRecordsInRange(ScanOpPtr scanPtr, Uint32* out)
+Dbtux::statRecordsInRange(ScanOpPtr scanPtr, Uint32* out, Uint32 out_words)
 {
   ScanOp& scan = *scanPtr.p;
   Frag& frag = *c_fragPool.getPtr(scan.m_fragPtrI);
@@ -87,12 +92,22 @@ Dbtux::statRecordsInRange(ScanOpPtr scanPtr, Uint32* out)
     const ScanBound& scanBound = scan.m_scanBound[idir];
     KeyDataC searchBoundData(index.m_keySpec, true);
     KeyBoundC searchBound(searchBoundData);
-    unpackBound(c_ctx, scanBound, searchBound);
-    searchToScan(frag, idir, searchBound, pos2);
+    unpackBound(c_ctx.c_searchKey, scanBound, searchBound);
+
+    KeyDataArray *searchKeyDataArray = new (&c_ctx.searchKeyDataArray)
+                                         KeyDataArray();
+    searchKeyDataArray->init_bound(searchBound, scanBound.m_cnt);
+    KeyBoundArray *searchBoundArray = new (&c_ctx.searchKeyBoundArray)
+      KeyBoundArray(&index.m_keySpec,
+                    &c_ctx.searchKeyDataArray,
+                    scanBound.m_side);
+
+    searchToScan(frag, idir, *searchBoundArray, pos2);
     // committed read (same timeslice) and range not empty
     ndbrequire(pos2.m_loc != NullTupLoc);
   }
   // wl4124_todo change all to Uint64 if ever needed (unlikely)
+  ndbassert(4 <= out_words);
   out[0] = (Uint32)frag.m_entryCount;
   out[2] = getEntriesBeforeOrAfter(frag, pos1, 0);
   out[3] = getEntriesBeforeOrAfter(frag, pos2, 1);
@@ -120,11 +135,11 @@ Uint32
 Dbtux::getEntriesBeforeOrAfter(Frag& frag, TreePos pos, unsigned idir)
 {
   NodeHandle node(frag);
-  selectNode(node, pos.m_loc);
+  selectNode(c_ctx, node, pos.m_loc);
   Uint16 path[MaxTreeDepth + 1];
   unsigned depth = getPathToNode(node, path);
   ndbrequire(depth != 0 && depth <= MaxTreeDepth);
-  TreeHead& tree = frag.m_tree;
+  // compiler warning unused: TreeHead& tree = frag.m_tree;
   Uint32 cnt = 0;
   Uint32 tot = (Uint32)frag.m_entryCount;
   unsigned i = 0;
@@ -169,7 +184,7 @@ Dbtux::getPathToNode(NodeHandle node, Uint16* path)
   unsigned i = MaxTreeDepth;
   while (loc != NullTupLoc) {
     jam();
-    selectNode(node, loc);
+    selectNode(c_ctx, node, loc);
     path[i] = node.getSide() | (node.getOccup() << 8);
     loc = node.getLink(2);
     ndbrequire(i != 0);
@@ -198,8 +213,11 @@ int
 Dbtux::statScanInit(StatOpPtr statPtr, const Uint32* data, Uint32 len,
                     Uint32* usedLen)
 {
+  ScanOpPtr scanPtr;
   StatOp& stat = *statPtr.p;
-  ScanOp& scan = *c_scanOpPool.getPtr(stat.m_scanOpPtrI);
+  scanPtr.i = stat.m_scanOpPtrI;
+  ndbrequire(c_scanOpPool.getValidPtr(scanPtr));
+  ScanOp& scan = *scanPtr.p;
   Frag& frag = *c_fragPool.getPtr(scan.m_fragPtrI);
   const Index& index = *c_indexPool.getPtr(scan.m_indexId);
   D("statScanInit");
@@ -287,7 +305,10 @@ int
 Dbtux::statScanAddRow(StatOpPtr statPtr, TreeEnt ent)
 {
   StatOp& stat = *statPtr.p;
-  ScanOp& scan = *c_scanOpPool.getPtr(stat.m_scanOpPtrI);
+  ScanOpPtr scanPtr;
+  scanPtr.i = stat.m_scanOpPtrI;
+  ndbrequire(c_scanOpPool.getValidPtr(scanPtr));
+  ScanOp& scan = *scanPtr.p;
   Frag& frag = *c_fragPool.getPtr(scan.m_fragPtrI);
   D("statScanAddRow" << V(stat));
 
@@ -343,7 +364,7 @@ Dbtux::statScanAddRow(StatOpPtr statPtr, TreeEnt ent)
   {
     NodeHandle node(frag);
     TreePos pos = scan.m_scanPos;
-    selectNode(node, pos.m_loc);
+    selectNode(c_ctx, node, pos.m_loc);
     // more entries in this node
     const unsigned occup = node.getOccup();
     // funny cast to avoid signed vs unsigned warning
@@ -363,7 +384,7 @@ Dbtux::statScanAddRow(StatOpPtr statPtr, TreeEnt ent)
     {
       jam();
       TupLoc loc = node.getLink(2);
-      selectNode(node, loc);
+      selectNode(c_ctx, node, loc);
     }
     // did not reach root
     if (node.getSide() != 2)
@@ -388,11 +409,21 @@ Dbtux::statScanAddRow(StatOpPtr statPtr, TreeEnt ent)
     stat.m_batchCurr = 0;
     return 1;
   }
+  /* Take a break to avoid problems with a long stretch of equal keys */
+  const Uint32 MaxAddRowsWithoutBreak = 16;
+  if (stat.m_rowCount % MaxAddRowsWithoutBreak == 0)
+  {
+    jam();
+    D("Taking a break from stat scan");
+    return 2; // Take a break
+  }
+
+  /* Iterate to next index entry */
   return 0;
 }
 
 void
-Dbtux::statScanReadKey(StatOpPtr statPtr, Uint32* out)
+Dbtux::statScanReadKey(StatOpPtr statPtr, Uint32* out, Uint32 out_words)
 {
   StatOp& stat = *statPtr.p;
   int ret;
@@ -402,11 +433,12 @@ Dbtux::statScanReadKey(StatOpPtr statPtr, Uint32* out)
   ndbrequire(ret == 0);
   D("statScanReadKey" << V(keyData));
   keyData.convert(NdbPack::Endian::Little);
+  ndbrequire(keyData.get_full_len() <= out_words * 4);
   memcpy(out, keyData.get_full_buf(), keyData.get_full_len());
 }
 
 void
-Dbtux::statScanReadValue(StatOpPtr statPtr, Uint32* out)
+Dbtux::statScanReadValue(StatOpPtr statPtr, Uint32* out, Uint32 out_words)
 {
   StatOp& stat = *statPtr.p;
   int ret;
@@ -435,6 +467,7 @@ Dbtux::statScanReadValue(StatOpPtr statPtr, Uint32* out)
 
   D("statScanReadValue" << V(valueData));
   valueData.convert(NdbPack::Endian::Little);
+  ndbrequire(valueData.get_full_len() <= out_words * 4);
   memcpy(out, valueData.get_full_buf(), valueData.get_full_len());
 }
 
@@ -447,13 +480,12 @@ Dbtux::execINDEX_STAT_REP(Signal* signal)
 
   switch (rep->requestType) {
   case IndexStatRep::RT_UPDATE_REQ:
-    ndbrequire(false);
-    break;
+    ndbabort();
   case IndexStatRep::RT_UPDATE_CONF:
     {
       Index& index = *c_indexPool.getPtr(rep->indexId);
       FragPtr fragPtr;
-      findFrag(index, rep->fragId, fragPtr);
+      findFrag(jamBuffer(), index, rep->fragId, fragPtr);
       ndbrequire(fragPtr.i != RNIL);
       // index.m_statFragPtrI need not be defined yet
       D("loadTime" << V(index.m_statLoadTime) << " ->" << V(rep->loadTime));
@@ -461,8 +493,7 @@ Dbtux::execINDEX_STAT_REP(Signal* signal)
     }
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 }
 
@@ -486,8 +517,7 @@ Dbtux::execINDEX_STAT_IMPL_REQ(Signal* signal)
     statMonStop(signal, mon);
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 }
 
@@ -498,22 +528,26 @@ Dbtux::statMonStart(Signal* signal, StatMon& mon)
   Index& index = *c_indexPool.getPtr(req->indexId);
   D("statMonStart" << V(mon));
 
-  // RT_START_MON also sends ZNIL to all non-monitoring nodes
-  if (req->fragId == ZNIL)
+  FragPtr fragPtr;
+  fragPtr.setNull();
+
+  if (req->fragId != ZNIL)
   {
     jam();
-    index.m_statFragPtrI = RNIL;
-    D("non-monitoring node");
+    findFrag(jamBuffer(), index, req->fragId, fragPtr);
+  }
+
+  if (fragPtr.i != RNIL)
+  {
+    jam();
+    index.m_statFragPtrI = fragPtr.i;
+    fragPtr.p->m_entryOps = 0;
+    D("monitoring node" << V(index));
   }
   else
   {
     jam();
-    FragPtr fragPtr;
-    findFrag(index, req->fragId, fragPtr);
-    ndbrequire(fragPtr.i != RNIL);
-    index.m_statFragPtrI = fragPtr.i;
-    fragPtr.p->m_entryOps = 0;
-    D("monitoring node" << V(index));
+    index.m_statFragPtrI = RNIL;
   }
 
   statMonConf(signal, mon);
