@@ -1,17 +1,25 @@
-/* Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+/*
+   Copyright (c) 2008, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <ndb_global.h>
 
@@ -24,14 +32,18 @@
 #include <signaldata/FsOpenReq.hpp>
 #include <signaldata/FsReadWriteReq.hpp>
 #include <signaldata/AllocMem.hpp>
+#include "util/require.h"
 #include "Ndbfs.hpp"
 #include <NdbSleep.h>
 
 #include <EventLogger.hpp>
-extern EventLogger * g_eventLogger;
+
+#define JAM_FILE_ID 388
+
 
 AsyncIoThread::AsyncIoThread(class Ndbfs& fs, bool bound)
-  : m_fs(fs)
+  : m_fs(fs),
+    m_real_time(false)
 {
   m_current_file = 0;
   if (bound)
@@ -60,13 +72,7 @@ struct NdbThread*
 AsyncIoThread::doStart()
 {
   // Stacksize for filesystem threads
-#if !defined(DBUG_OFF) && defined (__hpux)
-  // Empirical evidence indicates at least 32k
-  const NDB_THREAD_STACKSIZE stackSize = 32768;
-#else
-  // Otherwise an 8k stack should be enough
-  const NDB_THREAD_STACKSIZE stackSize = 8192;
-#endif
+  const NDB_THREAD_STACKSIZE stackSize = 128*1024;
 
   char buf[16];
   numAsyncFiles++;
@@ -118,6 +124,7 @@ void
 AsyncIoThread::dispatch(Request *request)
 {
   assert(m_current_file);
+  require(m_current_file->thread_bound());
   assert(m_current_file->getThread() == this);
   assert(theMemoryChannelPtr == &theMemoryChannel);
   theMemoryChannelPtr->writeChannel(request);
@@ -126,30 +133,76 @@ AsyncIoThread::dispatch(Request *request)
 void
 AsyncIoThread::run()
 {
+  bool first_flag = true;
   Request *request;
+  NDB_TICKS last_yield_ticks;
 
   // Create theMemoryChannel in the thread that will wait for it
   NdbMutex_Lock(theStartMutexPtr);
   theStartFlag = true;
-  NdbMutex_Unlock(theStartMutexPtr);
   NdbCondition_Signal(theStartConditionPtr);
+  NdbMutex_Unlock(theStartMutexPtr);
+
+  EmulatedJamBuffer jamBuffer;
+  jamBuffer.theEmulatedJamIndex = 0;
+  // This key is needed by jamNoBlock().
+  NDB_THREAD_TLS_JAM = &jamBuffer;
 
   while (1)
   {
+    if (m_real_time)
+    {
+      /**
+       * If we are running in real-time we'll simply insert a break every
+       * so often to ensure that low-prio threads aren't blocked from the
+       * CPU, this is especially important if we're using a compressed
+       * file system where lots of CPU is used by this thread.
+       */
+      bool yield_flag = false;
+      const NDB_TICKS current_ticks = NdbTick_getCurrentTicks();
+
+      if (first_flag)
+      {
+        first_flag = false;
+        yield_flag = true;
+      }
+      else
+      {
+        Uint64 micros_passed =
+          NdbTick_Elapsed(last_yield_ticks, current_ticks).microSec();
+        if (micros_passed > 10000)
+        {
+          yield_flag = true;
+        }
+      }
+      if (yield_flag)
+      {
+        if (NdbThread_yield_rt(theThreadPtr, true))
+        {
+          m_real_time = false;
+        }
+        last_yield_ticks = current_ticks;
+      }
+    }
     request = theMemoryChannelPtr->readChannel();
     if (!request || request->action == Request::end)
     {
-      DEBUG(ndbout_c("Nothing read from Memory Channel in AsyncFile"));
+      DEBUG(g_eventLogger->info("Nothing read from Memory Channel in AsyncFile"));
       theStartFlag = false;
       return;
     }//if
 
     AsyncFile * file = request->file;
+    /*
+     * Associate request with thread to be able to reuse encryption context
+     * m_openssl_evp_op.
+     */
+    request->thread = this;
     m_current_request= request;
     switch (request->action) {
     case Request::open:
       file->openReq(request);
-      if (request->error == 0 && request->m_do_bind)
+      if (request->error.code == 0 && request->m_do_bind)
         attach(file);
       break;
     case Request::close:
@@ -165,23 +218,13 @@ AsyncIoThread::run()
     case Request::read:
       file->readReq(request);
       break;
-    case Request::readv:
-      file->readvReq(request);
-      break;
     case Request::write:
       file->writeReq(request);
       break;
-    case Request::writev:
-      file->writevReq(request);
-      break;
     case Request::writeSync:
       file->writeReq(request);
       file->syncReq(request);
       break;
-    case Request::writevSync:
-      file->writevReq(request);
-      file->syncReq(request);
-      break;
     case Request::sync:
       file->syncReq(request);
       break;
@@ -224,7 +267,7 @@ AsyncIoThread::run()
         return;
       }
     default:
-      DEBUG(ndbout_c("Invalid Request"));
+      DEBUG(g_eventLogger->info("Invalid Request"));
       abort();
       break;
     }//switch
@@ -246,7 +289,7 @@ AsyncIoThread::allocMemReq(Request* request)
     bool memlock = !!(request->par.alloc.requestInfo & AllocMemReq::RT_MEMLOCK);
     request->par.alloc.ctx->m_mm.map(&watchDog, memlock);
     request->par.alloc.bytes = 0;
-    request->error = 0;
+    NDBFS_SET_REQUEST_ERROR(request, 0);
     break;
   }
   case AllocMemReq::RT_EXTEND:
@@ -255,7 +298,7 @@ AsyncIoThread::allocMemReq(Request* request)
      */
     assert(false);
     request->par.alloc.bytes = 0;
-    request->error = 1;
+    NDBFS_SET_REQUEST_ERROR(request, 1);
     break;
   }
 }
@@ -263,11 +306,24 @@ AsyncIoThread::allocMemReq(Request* request)
 void
 AsyncIoThread::buildIndxReq(Request* request)
 {
+  /**
+   * Rebind thread config to allow different behaviour
+   * during Index build.
+   */
+  THRConfigRebinder idxbuild_cpulock(&m_fs.m_ctx.m_config.m_thr_config,
+                                     THRConfig::T_IXBLD,
+                                     theThreadPtr);
+
   mt_BuildIndxReq req;
   memcpy(&req, &request->par.build.m_req, sizeof(req));
-  req.mem_buffer = request->file->m_page_ptr.p;
-  req.buffer_size = request->file->m_page_cnt * sizeof(GlobalPage);
-  request->error = (* req.func_ptr)(&req);
+  Uint32 rg;
+  Ptr<GlobalPage> ptr;
+  Uint32 cnt;
+  bool has_buffer = request->file->get_buffer(rg, ptr, cnt);
+  require(has_buffer);
+  req.mem_buffer = ptr.p;
+  req.buffer_size = cnt * sizeof(GlobalPage);
+  NDBFS_SET_REQUEST_ERROR(request, (* req.func_ptr)(&req));
 }
 
 void
@@ -278,7 +334,6 @@ AsyncIoThread::attach(AsyncFile* file)
   m_current_file = file;
   theMemoryChannelPtr = &theMemoryChannel;
   file->attach(this);
-  m_fs.cnt_active_bound(1);
 }
 
 void
@@ -286,7 +341,7 @@ AsyncIoThread::detach(AsyncFile* file)
 {
   if (m_current_file == 0)
   {
-    assert(file->getThread() == 0);
+    assert(!file->thread_bound());
   }
   else
   {
@@ -295,6 +350,35 @@ AsyncIoThread::detach(AsyncFile* file)
     m_current_file = 0;
     theMemoryChannelPtr = &m_fs.theToBoundThreads;
     file->detach(this);
-    m_fs.cnt_active_bound(-1);
+  }
+}
+
+
+const char*
+Request::actionName(Request::Action action)
+{
+  switch(action){
+  case Request::open:
+    return "open";
+  case Request::close:
+    return "close";
+  case Request::closeRemove:
+    return "closeRemove";
+  case Request::read:
+    return "read";
+  case Request::write:
+    return "write";
+  case Request::writeSync:
+    return "writeSync";
+  case Request::sync:
+    return "sync";
+  case Request::end:
+    return "end";
+  case Request::append:
+    return "append";
+  case Request::rmrf:
+    return "rmrf";
+  default:
+    return "Unknown action";
   }
 }
