@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -21,14 +28,29 @@
 #include <SimulatedBlock.hpp>
 
 #include <DLCHashTable.hpp>
-#include <DLCFifoList.hpp>
+#include <IntrusiveList.hpp>
 #include <NodeBitmask.hpp>
 #include <signaldata/LCP.hpp>
+#include <signaldata/RedoStateRep.hpp>
 #include "lgman.hpp"
 
+#include <EventLogger.hpp>
 #include <NdbOut.hpp>
 #include <OutputStream.hpp>
 
+#define JAM_FILE_ID 462
+
+/**
+ * ndbd has only one pgman sharing the single disk page buffer between
+ * the locked extent pages and the dynamic caching of data pages from
+ * data files.  Limit the extent page usage to 25% of the buffer. One
+ * extent page can describe 1022 extents.  25% of the default
+ * DiskPageBufferMemory of 64 MB (512 pages) will be sufficient to
+ * open a single data file of max recommended size 32GB, needing 33
+ * extent pages.
+ */
+#define NDBD_EXTENT_PAGE_PERCENT 25
+
 /*
  * PGMAN
  *
@@ -202,7 +224,7 @@
  *
  * 4. The "LCP loop".  Started periodically by NDB.  Each loop starts
  * pageout of a number of hash list entries.  Continues via delay signal
- * until done.
+ * until done. TODO RONM: Describe new per fragment LCP loop.
  *
  * SPECIAL CASES
  *
@@ -212,14 +234,14 @@
  * A TUP scan is likely to access a page repeatedly within a short time.
  * This can make the page hot when it should not be.  Such "correlated
  * requests" are handled by a request flag which modifies default LIRS
- * processing.  [fill in details later]
+ * processing.  [fill in details later, currently not implemented, TODO]
  *
  * Also PK operations make 2 rapid page references.  The 2nd one is for
  * commit.  This too should be handled as a correlated request.
  *
  * CLIENT TSMAN
  *
- * TSMAN reads "meta" pages such as extent headers.  There are currently
+ * TSMAN reads "meta" pages such as extent headers.  These are currently
  * "locked" forever in PGMAN cache.
  *
  * CLIENT DBTUP
@@ -238,19 +260,38 @@ class Pgman : public SimulatedBlock
 {
 public:
   Pgman(Block_context& ctx, Uint32 instanceNumber = 0);
-  virtual ~Pgman();
+  ~Pgman() override;
+
+  /* Special function to indicate the block is the extra PGMAN worker */
+  void init_extra_pgman();
+
   BLOCK_DEFINES(Pgman);
 
 private:
+  friend class Tsman;
   friend class Page_cache_client;
   friend class PgmanProxy;
 
   struct Page_entry; // CC
   friend struct Page_entry;
 
+  struct File_entry
+  {
+    File_entry(){}
+
+    Uint32 m_magic;
+    Uint32 m_fd;
+    Uint32 m_ndb_version;
+    Uint32 m_file_no;
+    Uint32 nextPool;
+  };
+  typedef RecordPool<RWPool<File_entry> > File_entry_pool;
+  File_entry_pool m_file_entry_pool;
+
   struct Page_request {
     enum Flags {
       OP_MASK       = 0x000F // 4 bits for TUP operation
+      ,UNDO_GET_REQ = 0x0010 // Get page to get table id and fragment id
       ,LOCK_PAGE    = 0x0020 // lock page in memory
       ,EMPTY_PAGE   = 0x0040 // empty (new) page
       ,ALLOC_REQ    = 0x0080 // part of alloc
@@ -261,45 +302,44 @@ private:
 #ifdef ERROR_INSERT
       ,DELAY_REQ    = 0x1000 // Force request to be delayed
 #endif
+      ,UNDO_REQ     = 0x2000 // Request from UNDO processing
+      ,DISK_SCAN    = 0x4000 // Request from Disk scan
+      ,ABORT_REQ    = 0x8000 // Part of ABORT will not update LSN
+      ,COPY_FRAG    = 0x10000// Request part of BACKUP/COPY_FRAG processing
     };
     
-    Uint16 m_block; // includes instance
-    Uint16 m_flags;
+    Uint32 m_block; // includes instance
+    Uint32 m_flags;
     SimulatedBlock::Callback m_callback;
 
 #ifdef ERROR_INSERT
-    Uint64 m_delay_until_time;
+    NDB_TICKS m_delay_until_time;
 #endif
     Uint32 nextList;
     Uint32 m_magic;
+    NDB_TICKS m_start_time;
   };
 
-  typedef RecordPool<Page_request, WOPool> Page_request_pool;
-  typedef SLFifoListImpl<Page_request_pool, Page_request> Page_request_list;
-  typedef LocalSLFifoListImpl<Page_request_pool, Page_request> Local_page_request_list;
+  typedef RecordPool<WOPool<Page_request> > Page_request_pool;
+  typedef SLFifoList<Page_request_pool> Page_request_list;
+  typedef LocalSLFifoList<Page_request_pool> Local_page_request_list;
   
-  struct Page_entry_stack_ptr {
-    Uint32 nextList;
-    Uint32 prevList;
-  };
-
-  struct Page_entry_queue_ptr {
-    Uint32 nextList;
-    Uint32 prevList;
-  };
-
-  struct Page_entry_sublist_ptr {
-    Uint32 nextList;
-    Uint32 prevList;
-  };
-
-  typedef Uint16 Page_state;
+  typedef Uint32 Page_state;
   
-  struct Page_entry : Page_entry_stack_ptr,
-                      Page_entry_queue_ptr,
-                      Page_entry_sublist_ptr {
+  enum DirtyState {
+    IN_FIRST_FRAG_DIRTY_LIST = 0,
+    IN_SECOND_FRAG_DIRTY_LIST = 1,
+    IN_LCP_OUT_LIST = 2,
+    IN_NO_DIRTY_LIST = 3
+  };
+
+  struct Page_entry
+  {
     Page_entry() {}
-    Page_entry(Uint32 file_no, Uint32 page_no);
+    Page_entry(Uint32 file_no,
+               Uint32 page_no,
+               Uint32 tableId,
+               Uint32 fragmentId);
 
     enum State {
       NO_STATE = 0x0000
@@ -318,6 +358,8 @@ private:
       ,HOT     = 0x2000 // page is hot
       ,ONSTACK = 0x4000 // page is on LIRS stack
       ,ONQUEUE = 0x8000 // page is on LIRS queue
+      ,WAIT_LCP= 0x10000 //BUSY page holding up LCP
+      ,PREP_LCP= 0x20000 //Page is flushed as part of prepare LCP
     };
     
     enum Sublist {
@@ -334,13 +376,20 @@ private:
     };
 
     Uint16 m_file_no;       // disk page address set at seize
+
+    DirtyState m_dirty_state;
+
+    bool m_dirty_during_pageout;
+
     Page_state m_state;         // flags (0 for new entry)
-    
+ 
     Uint32 m_page_no;
     Uint32 m_real_page_i;
     Uint64 m_lsn;
-    
-    Uint32 m_last_lcp;
+
+    Uint32 m_table_id;
+    Uint32 m_fragment_id;
+
     Uint32 m_dirty_count;
     Uint32 m_copy_page_i;
     union {
@@ -350,6 +399,18 @@ private:
     
     Page_request_list::Head m_requests;
     
+    Uint32 nextStack;
+    Uint32 prevStack;
+
+    Uint32 nextQueue;
+    Uint32 prevQueue;
+
+    Uint32 nextSublist;
+    Uint32 prevSublist;
+
+    Uint32 nextDirty;
+    Uint32 prevDirty;
+
     Uint32 nextHash;
     Uint32 prevHash;
     
@@ -362,47 +423,280 @@ private:
 #ifdef VM_TRACE
     Pgman* m_this;
 #endif
+    Uint64 m_time_tracking;
   };
 
-  typedef DLCHashTable<Page_entry> Page_hashlist;
-  typedef DLCFifoList<Page_entry, Page_entry_stack_ptr> Page_stack;
-  typedef DLCFifoList<Page_entry, Page_entry_queue_ptr> Page_queue;
-  typedef DLCFifoList<Page_entry, Page_entry_sublist_ptr> Page_sublist;
+  typedef ArrayPool<Page_entry> Page_entry_pool;
+  typedef DLCHashTable<Page_entry_pool> Page_hashlist;
+  typedef DLCFifoList<Page_entry_pool, IA_Stack> Page_stack;
+  typedef DLCFifoList<Page_entry_pool, IA_Queue> Page_queue;
+  typedef DLCFifoList<Page_entry_pool, IA_Sublist> Page_sublist;
+  typedef DLCFifoList<Page_entry_pool, IA_Dirty> Page_dirty_list;
+  typedef LocalDLCFifoList<Page_entry_pool, IA_Dirty> LocalPage_dirty_list;
+
+  /**
+   * We keep all page entries in a linked list on the fragment record.
+   * This is only used for LCP processing. When the LCP for a fragment
+   * starts we will move the linked list to the m_dirty_list_lcp list.
+   * When a page is sent to disk as part of LCP it is moved to the
+   * m_dirty_list_lcp_out list.
+   *
+   * When both of those lists are empty we are done with the LCP for
+   * the fragment.
+   *
+   * LCP of extent pages is done by simply ensuring that no pages
+   * in the extent page list is dirty. When this is completed the
+   * LCP of extent pages is done.
+   */
+  struct FragmentRecord
+  {
+    FragmentRecord(Pgman &pgman, Uint32, Uint32);
+    Uint32 m_table_id;
+    Uint32 m_fragment_id;
+
+    Page_dirty_list::Head m_dirty_list;
+
+    DirtyState m_current_lcp_dirty_state;
+
+    bool m_is_frag_ready_for_prep_lcp_writes;
+    Uint32 prevList;
+    Uint32 nextList;
+    Uint32 prevHash;
+    union
+    {
+      Uint32 nextPool;
+      Uint32 nextHash;
+    };
+
+    inline bool equal(const FragmentRecord & p) const
+    {
+      return (p.m_table_id == m_table_id &&
+              p.m_fragment_id == m_fragment_id);
+    }
+    inline Uint32 hashValue() const
+    {
+      /**
+       * tableId is fairly good, usually there are 2 fragments per
+       * LDM per table. So we provide a function that gives at least
+       * sometimes a different first bit dependent on fragmentId.
+       */
+      return ((m_table_id << 1) +
+              ((m_fragment_id >> 1) & 1));
+    }
+  };
+  typedef Ptr<FragmentRecord> FragmentRecordPtr;
+  typedef ArrayPool<FragmentRecord> FragmentRecord_pool;
+  FragmentRecord_pool m_fragmentRecordPool;
+#define NUM_ORDERED_LISTS 128
+  typedef LocalDLFifoList<FragmentRecord_pool> Local_FragmentRecord_list;
+  DLFifoList<FragmentRecord_pool>::Head
+    m_fragmentRecordList[NUM_ORDERED_LISTS];
+  void insert_ordered_fragment_list(FragmentRecordPtr);
+  bool get_first_ordered_fragment(FragmentRecordPtr&);
+  bool get_next_ordered_fragment(FragmentRecordPtr&);
+  Uint32 get_ordered_list_from_table_id(Uint32 table_id);
+
+  DLHashTable<FragmentRecord_pool, FragmentRecord> m_fragmentRecordHash;
+
+  struct TableRecord
+  {
+    bool m_is_table_ready_for_prep_lcp_writes;
+    Uint32 m_num_prepare_lcp_outstanding;
+    Uint32 nextPool;
+  };
+  typedef Ptr<TableRecord> TableRecordPtr;
+  typedef ArrayPool<TableRecord> TableRecord_pool;
+  TableRecord_pool m_tableRecordPool;
+public:
+  void set_table_ready_for_prep_lcp_writes(Uint32, bool);
+  bool is_prep_lcp_writes_outstanding(Uint32);
+
+private:
+  Page_dirty_list m_dirty_list_lcp;
+  Page_dirty_list m_dirty_list_lcp_out;
+
+  Uint32 m_lcp_table_id;
+  Uint32 m_lcp_fragment_id;
+
+  Uint32 m_prev_lcp_table_id;
+  Uint32 m_prev_lcp_fragment_id;
+  bool m_lcp_loop_ongoing;
+  bool m_lcp_ongoing;
+  Uint32 m_num_ldm_completed_lcp;
+  Uint32 m_locked_pages_written;
+  Uint32 m_lcp_outstanding;     // remaining i/o waits
+  Uint32 m_prep_lcp_outstanding;
+  SyncExtentPagesReq::LcpOrder m_sync_extent_order;
+  bool m_sync_extent_pages_ongoing;
+  bool m_sync_extent_continueb_ongoing;
+  Uint32 m_sync_extent_next_page_entry;
+  SyncPageCacheReq m_sync_page_cache_req;
+  SyncExtentPagesReq m_sync_extent_pages_req;
+  EndLcpReq m_end_lcp_req;
+
+  /* Methods to handle local LCP from LGMAN after UNDO log execution */
+  void sendSYNC_PAGE_WAIT_REP(Signal *signal, bool normal_pages);
+  void sendSYNC_PAGE_CACHE_REQ(Signal*, FragmentRecordPtr);
+  void sendSYNC_EXTENT_PAGES_REQ(Signal*);
+  void sendEND_LCPCONF(Signal*);
+
+  void check_restart_lcp(Signal*, bool check_prepare_lcp);
+  void start_lcp_loop(Signal*);
+  void handle_lcp(Signal*, Uint32 tableId, Uint32 fragmentId);
+  void handle_lcp(Signal*, FragmentRecord*);
+  void handle_prepare_lcp(Signal*, FragmentRecordPtr);
+  void finish_lcp(Signal*, FragmentRecord*);
+  void finish_sync_extent_pages(Signal*);
+  Uint32 get_num_lcp_pages_to_write(bool);
+
+  void process_lcp_locked(Signal* signal, Ptr<Page_entry> ptr);
+  void process_lcp_locked_fswriteconf(Signal* signal, Ptr<Page_entry> ptr);
+  void copy_back_page(Ptr<Page_entry> ptr);
+
+  /**
+   * In ndbmtd, there is an extra pgman instance not associated with
+   * an LDM, which is mostly used for accessing extent pages
+   * containing metadata about extents.
+   */
+  bool m_extra_pgman;
+  /**
+   * extra_pgman reserves 'm_extra_pgman_reserve' slots of the disk
+   * page buffer memory for undo log execution during restart. These
+   * will be used to read in data pages in order to find the tableid
+   * and fragmentid of an undo log record (in order to decide the ldm
+   * that will undo this log record), if they cannot be retrieved from
+   * tsman extent info.  These pages can be evicted after retrieving
+   * table/fragids, and not given to the ldm for undo. The ldm will
+   * read the page itself into its share of the disk page buffer memory.
+   */
+  Uint32 m_extra_pgman_reserve_pages;
 
   class Dbtup *c_tup;
   class Lgman *c_lgman;
   class Tsman *c_tsman;
+  class Backup *c_backup;
 
   // loop status
   bool m_stats_loop_on;
   bool m_busy_loop_on;
   bool m_cleanup_loop_on;
 
-  // LCP variables
-  enum LCP_STATE
-  {
-    LS_LCP_OFF = 0
-    ,LS_LCP_ON = 1
-    ,LS_LCP_MAX_LCP_OUTSTANDING = 2
-    ,LS_LCP_LOCKED = 3
-  } m_lcp_state;
-  Uint32 m_last_lcp;
-  Uint32 m_last_lcp_complete;
-  Uint32 m_lcp_curr_bucket;
-  Uint32 m_lcp_outstanding;     // remaining i/o waits
-  EndLcpReq m_end_lcp_req;
-  
+  // LCP counters
+  Uint64 m_tot_pages_made_dirty;
+  Uint64 m_current_lcp_pageouts;
+  Uint64 m_current_lcp_flushes;
+  Uint64 m_last_flushes;
+  Uint64 m_start_lcp_made_dirty;
+  Uint64 m_last_lcp_made_dirty;
+  Uint64 m_dirty_page_rate_per_sec;
+  Uint64 m_last_pageouts;
+  Uint64 m_last_made_dirty;
+
+  Uint64 m_num_dirty_pages;
+  Uint64 m_available_lcp_pageouts;
+  Uint64 m_prep_available_lcp_pageouts;
+  Uint64 m_available_lcp_pageouts_used;
+
+  NDB_TICKS m_last_track_lcp_speed_call;
+  NDB_TICKS m_lcp_start_time;
+
+  Uint64 m_lcp_dd_percentage;
+  Uint64 m_max_lcp_pages_outstanding;
+  Uint64 m_prep_max_lcp_pages_outstanding;
+
+  Uint64 m_lcp_time_in_ms;
+  Uint64 m_mm_curr_disk_write_speed;
+  Uint64 m_percent_spent_in_checkpointing;
+  Uint64 m_max_pageout_rate;
+
+  bool m_track_lcp_speed_loop_ongoing;
+public:
+  bool lcp_end_point(Uint32 lcp_time_in_ms, bool first, bool internal);
+  void set_lcp_dd_percentage(Uint32 dd_percentage);
+  void set_current_disk_write_speed(Uint64);
+  void lcp_start_point(Signal*, Uint32, Uint32);
+private:
+  void do_track_handle_lcp_speed_loop(Signal*);
+  Uint64 get_current_lcp_made_dirty();
+
+#define PGMAN_TIME_TRACK_NUM_RANGES 20
+  Uint64 m_time_track_histogram_upper_bound[PGMAN_TIME_TRACK_NUM_RANGES];
+  Uint64 m_time_track_reads[PGMAN_TIME_TRACK_NUM_RANGES];
+  Uint64 m_time_track_writes[PGMAN_TIME_TRACK_NUM_RANGES];
+  Uint64 m_time_track_log_waits[PGMAN_TIME_TRACK_NUM_RANGES];
+  Uint64 m_time_track_get_page[PGMAN_TIME_TRACK_NUM_RANGES];
+
+  Uint64 m_pages_made_dirty;
+  Uint64 m_reads_issued;
+  Uint64 m_reads_completed;
+  Uint64 m_writes_issued;
+  Uint64 m_writes_completed;
+  Uint64 m_tot_writes_completed;
+  Uint64 m_log_writes_issued;
+  Uint64 m_log_writes_completed;
+  Uint64 m_get_page_calls_issued;
+  Uint64 m_get_page_reqs_issued;
+  Uint64 m_get_page_reqs_completed;
+
+#define NUM_STAT_HISTORY 20
+  Uint32 m_pages_made_dirty_history[NUM_STAT_HISTORY];
+  Uint32 m_reads_issued_history[NUM_STAT_HISTORY];
+  Uint32 m_reads_completed_history[NUM_STAT_HISTORY];
+  Uint32 m_writes_issued_history[NUM_STAT_HISTORY];
+  Uint32 m_writes_completed_history[NUM_STAT_HISTORY];
+  Uint32 m_log_writes_issued_history[NUM_STAT_HISTORY];
+  Uint32 m_log_writes_completed_history[NUM_STAT_HISTORY];
+  Uint32 m_get_page_calls_issued_history[NUM_STAT_HISTORY];
+  Uint32 m_get_page_reqs_issued_history[NUM_STAT_HISTORY];
+  Uint32 m_get_page_reqs_completed_history[NUM_STAT_HISTORY];
+  Uint32 m_stat_time_delay[NUM_STAT_HISTORY];
+
+  Uint32 m_last_stat_index;
+  Uint32 m_max_dd_latency_ms;
+  Uint32 m_dd_using_same_disk;
+  RedoStateRep::RedoAlertState m_redo_alert_state;
+  RedoStateRep::RedoAlertState m_redo_alert_state_last_lcp;
+  Uint32 m_raise_redo_alert_state;
+  Uint64 m_redo_alert_factor;
+  Uint64 m_last_time_calc_stats_loop;
+
+  Uint64 m_num_dd_accesses;
+  Uint64 m_total_dd_latency_us;
+  Uint64 m_total_write_latency_us;
+  Uint64 m_last_lcp_writes_completed;
+  Uint64 m_last_lcp_total_write_latency_us;
+  Uint64 m_last_lcp_write_latency_us;
+
+  Uint64 m_outstanding_dd_requests;
+
+  Uint32 m_abort_counter;
+  Uint32 m_abort_level;
+
+public:
+  void set_redo_alert_state(RedoStateRep::RedoAlertState new_state);
+private:
+  void do_calc_stats_loop(Signal*);
+  bool check_overload_error();
+
+  void add_histogram(Uint64 elapsed_time, Uint64 *histogram);
+  void handle_reads_time_tracking(Ptr<Page_entry>);
+  void handle_writes_time_tracking(Ptr<Page_entry>);
+  void handle_log_waits_time_tracking(Ptr<Page_entry>);
+
   // clean-up variables
   Ptr<Page_entry> m_cleanup_ptr;
- 
+
+  NdbMutex *m_access_extent_page_mutex;
+  void lock_access_extent_page();
+  void unlock_access_extent_page();
   // file map
-  typedef DataBuffer<15> File_map;
+  typedef DataBuffer<15,ArrayPool<DataBufferSegment<15> > > File_map;
   File_map m_file_map;
   File_map::DataBufferPool m_data_buffer_pool;
 
   // page entries and requests
   Page_request_pool m_page_request_pool;
-  ArrayPool<Page_entry> m_page_entry_pool;
+  Page_entry_pool m_page_entry_pool;
   Page_hashlist m_page_hashlist;
   Page_stack m_page_stack;
   Page_queue m_page_queue;
@@ -418,7 +712,6 @@ private:
     Uint32 m_max_io_waits;
     Uint32 m_stats_loop_delay;
     Uint32 m_cleanup_loop_delay;
-    Uint32 m_lcp_loop_delay;
   } m_param;
 
   // runtime sizes and statistics
@@ -435,7 +728,9 @@ private:
       m_log_waits(0),
       m_page_requests_direct_return(0),
       m_page_requests_wait_q(0),
-      m_page_requests_wait_io(0)
+      m_page_requests_wait_io(0),
+      m_entries_high(0),
+      m_num_locked_pages(0)
     {}
     Uint32 m_num_pages;         // current number of cache pages
     Uint32 m_num_hot_pages;
@@ -449,6 +744,8 @@ private:
     Uint64 m_page_requests_direct_return;
     Uint64 m_page_requests_wait_q;
     Uint64 m_page_requests_wait_io;
+    Uint32 m_entries_high;
+    Uint32 m_num_locked_pages;
   } m_stats;
 
   enum CallbackIndex {
@@ -465,8 +762,11 @@ protected:
   void execREAD_CONFIG_REQ(Signal* signal);
   void execCONTINUEB(Signal* signal);
 
-  void execLCP_FRAG_ORD(Signal*);
-  void execEND_LCP_REQ(Signal*);
+  void execEND_LCPREQ(Signal*);
+  void execSYNC_PAGE_CACHE_REQ(Signal*);
+  void execSYNC_PAGE_CACHE_CONF(Signal*);
+  void execSYNC_EXTENT_PAGES_REQ(Signal*);
+  void execSYNC_EXTENT_PAGES_CONF(Signal*);
   void execRELEASE_PAGES_REQ(Signal*);
   
   void execFSREADCONF(Signal*);
@@ -479,58 +779,87 @@ protected:
   void execDATA_FILE_ORD(Signal*);
 
   void execDBINFO_SCANREQ(Signal*);
+public:
+  bool idle_fragment_lcp(Uint32 tableId, Uint32 fragmentId); 
 
 private:
   static Uint32 get_sublist_no(Page_state state);
-  void set_page_state(Ptr<Page_entry> ptr, Page_state new_state);
+  void set_page_state(EmulatedJamBuffer* jamBuf, Ptr<Page_entry> ptr,
+                      Page_state new_state);
 
   bool seize_cache_page(Ptr<GlobalPage>& gptr);
   void release_cache_page(Uint32 i);
 
   bool find_page_entry(Ptr<Page_entry>&, Uint32 file_no, Uint32 page_no);
-  Uint32 seize_page_entry(Ptr<Page_entry>&, Uint32 file_no, Uint32 page_no);
-  bool get_page_entry(Ptr<Page_entry>&, Uint32 file_no, Uint32 page_no);
-  void release_page_entry(Ptr<Page_entry>&);
+  Uint32 seize_page_entry(Ptr<Page_entry>&,
+                          Uint32 file_no,
+                          Uint32 page_no,
+                          Uint32 tableId,
+                          Uint32 fragmentId,
+                          EmulatedJamBuffer *jamBuf);
+  bool get_page_entry(EmulatedJamBuffer* jamBuf,
+                      Ptr<Page_entry>&, 
+                      Uint32 file_no,
+                      Uint32 page_no,
+                      Uint32 tableId,
+                      Uint32 fragmentId,
+                      Uint32 flags);
+  void release_page_entry(Ptr<Page_entry>&, EmulatedJamBuffer *jamBuf);
 
-  void lirs_stack_prune();
-  void lirs_stack_pop();
-  void lirs_reference(Ptr<Page_entry> ptr);
+  void lirs_stack_prune(EmulatedJamBuffer*);
+  void lirs_stack_pop(EmulatedJamBuffer*);
+  void lirs_reference(EmulatedJamBuffer* jamBuf, Ptr<Page_entry> ptr);
 
   void do_stats_loop(Signal*);
-  void do_busy_loop(Signal*, bool direct = false);
+  void do_busy_loop(Signal*, bool direct, EmulatedJamBuffer *jamBuf);
   void do_cleanup_loop(Signal*);
-  void do_lcp_loop(Signal*);
 
-  bool process_bind(Signal*);
-  bool process_bind(Signal*, Ptr<Page_entry> ptr);
-  bool process_map(Signal*);
-  bool process_map(Signal*, Ptr<Page_entry> ptr);
-  bool process_callback(Signal*);
-  bool process_callback(Signal*, Ptr<Page_entry> ptr);
+  bool process_bind(Signal*, EmulatedJamBuffer*);
+  bool process_bind(Signal*, Ptr<Page_entry> ptr, EmulatedJamBuffer*);
+  bool process_map(Signal*, EmulatedJamBuffer*);
+  bool process_map(Signal*, Ptr<Page_entry> ptr, EmulatedJamBuffer*);
+  bool process_callback(Signal*, EmulatedJamBuffer*);
+  bool process_callback(Signal*, Ptr<Page_entry> ptr, EmulatedJamBuffer*);
 
   bool process_cleanup(Signal*);
-  void move_cleanup_ptr(Ptr<Page_entry> ptr);
+  void move_cleanup_ptr(Ptr<Page_entry> ptr, EmulatedJamBuffer*);
 
-  LCP_STATE process_lcp(Signal*);
-  void process_lcp_locked(Signal* signal, Ptr<Page_entry> ptr);
-  void process_lcp_locked_fswriteconf(Signal* signal, Ptr<Page_entry> ptr);
-
-  void pagein(Signal*, Ptr<Page_entry>);
+  void pagein(Signal*, Ptr<Page_entry>, EmulatedJamBuffer *jamBuf);
   void fsreadreq(Signal*, Ptr<Page_entry>);
   void fsreadconf(Signal*, Ptr<Page_entry>);
-  void pageout(Signal*, Ptr<Page_entry>);
+  void pageout(Signal*, Ptr<Page_entry>, bool check_sync_lsn = true);
   void logsync_callback(Signal*, Uint32 ptrI, Uint32 res);
   void fswritereq(Signal*, Ptr<Page_entry>);
   void fswriteconf(Signal*, Ptr<Page_entry>);
 
-  int get_page_no_lirs(Signal*, Ptr<Page_entry>, Page_request page_req);
-  int get_page(Signal*, Ptr<Page_entry>, Page_request page_req);
-  void update_lsn(Ptr<Page_entry>, Uint32 block, Uint64 lsn);
-  Uint32 create_data_file();
-  Uint32 alloc_data_file(Uint32 file_no);
+  int get_page_no_lirs(EmulatedJamBuffer* jamBuf, Signal*, Ptr<Page_entry>, 
+                       Page_request page_req);
+  int get_page(EmulatedJamBuffer* jamBuf,
+               Signal*,
+               Ptr<Page_entry>, 
+               Page_request page_req);
+  Uint32 get_extent_page(EmulatedJamBuffer* jamBuf,
+                         Signal*,
+                         Ptr<Page_entry>, 
+                         Page_request page_req);
+  void set_lsn(Ptr<Page_entry>, Uint64 lsn);
+  void update_lsn(Signal *signal,
+                  EmulatedJamBuffer* jamBuf,
+                  Ptr<Page_entry>,
+                  Uint32 block, 
+                  Uint64 lsn);
+  int add_fragment(Uint32 tableId, Uint32 fragmentId);
+  void drop_fragment(Uint32 tableId, Uint32 fragmentId);
+  void insert_fragment_dirty_list(Ptr<Page_entry>,
+                                  Page_state,
+                                  EmulatedJamBuffer*);
+  void remove_fragment_dirty_list(Signal*, Ptr<Page_entry>, Page_state);
+  Uint32 create_data_file(Uint32 version);
+  Uint32 alloc_data_file(Uint32 file_no, Uint32 version);
   void map_file_no(Uint32 file_no, Uint32 fd);
   void free_data_file(Uint32 file_no, Uint32 fd = RNIL);
-  int drop_page(Ptr<Page_entry>);
+  int drop_page(Ptr<Page_entry>, EmulatedJamBuffer *jamBuf);
+  bool extent_pages_available(Uint32 pages_needed);
   
 #ifdef VM_TRACE
   bool debugFlag;        // not yet in use in 7.0
@@ -543,6 +872,8 @@ private:
   static const char* get_sublist_name(Uint32 list_no);
   friend class NdbOut& operator<<(NdbOut&, Ptr<Page_request>);
   friend class NdbOut& operator<<(NdbOut&, Ptr<Page_entry>);
+  friend void print(EventLogger *logger, Ptr<Pgman::Page_request> ptr);
+  friend void print(EventLogger *logger, Ptr<Pgman::Page_entry> ptr);
 };
 
 class NdbOut& operator<<(NdbOut&, Ptr<Pgman::Page_request>);
@@ -554,17 +885,20 @@ class Page_cache_client
   Uint32 m_block; // includes instance
   class PgmanProxy* m_pgman_proxy; // set if we go via proxy
   Pgman* m_pgman;
+  EmulatedJamBuffer* const m_jamBuf;
   DEBUG_OUT_DEFINES(PGMAN);
 
 public:
   Page_cache_client(SimulatedBlock* block, SimulatedBlock* pgman);
 
   struct Request {
+    Uint32 m_table_id;
+    Uint32 m_fragment_id;
     Local_key m_page;
     SimulatedBlock::Callback m_callback;
     
 #ifdef ERROR_INSERT
-    Uint64 m_delay_until_time;
+    NDB_TICKS m_delay_until_time;
 #endif
   };
 
@@ -581,6 +915,11 @@ public:
 #ifdef ERROR_INSERT
     ,DELAY_REQ = Pgman::Page_request::DELAY_REQ
 #endif
+    ,UNDO_REQ = Pgman::Page_request::UNDO_REQ
+    ,DISK_SCAN = Pgman::Page_request::DISK_SCAN
+    ,ABORT_REQ = Pgman::Page_request::ABORT_REQ
+    ,UNDO_GET_REQ = Pgman::Page_request::UNDO_GET_REQ
+    ,COPY_FRAG = Pgman::Page_request::COPY_FRAG
   };
   
   /**
@@ -593,7 +932,23 @@ public:
    */
   int get_page(Signal*, Request&, Uint32 flags);
 
-  void update_lsn(Local_key, Uint64 lsn);
+  /**
+   * Get an extent page
+   * Given that these pages are always locked in memory this function
+   * cannot return any failures, it will crash if it fails.
+   */
+  void get_extent_page(Signal*, Request&, Uint32 flags);
+
+  /**
+   * When reading the UNDO log we don't have access to the table id and
+   * fragment id, so to make sure that the table id and fragment id is
+   * properly set on the page entry object we use this method to set
+   * table id and fragment id on the page entry object.
+   */
+  bool init_page_entry(Request&);
+
+  void set_lsn(Local_key, Uint64 lsn);
+  void update_lsn(Signal*, Local_key, Uint64 lsn);
 
   /**
    * Drop page
@@ -603,16 +958,22 @@ public:
    *         >0 is ok
    */
   int drop_page(Local_key, Uint32 page_id);
-  
+
+  /**
+   * Check whether there are 'pages_needed' pages available
+   * to be used as extent pages by the extra_pgman.
+   */
+  bool extent_pages_available(Uint32 pages_needed);
+
   /**
    * Create file record
    */
-  Uint32 create_data_file(Signal*);
+  Uint32 create_data_file(Signal*, Uint32 version);
 
   /**
    * Alloc datafile record
    */
-  Uint32 alloc_data_file(Signal*, Uint32 file_no);
+  Uint32 alloc_data_file(Signal*, Uint32 file_no, Uint32 version);
 
   /**
    * Map file_no to m_fd
@@ -623,6 +984,19 @@ public:
    * Free file
    */
   void free_data_file(Signal*, Uint32 file_no, Uint32 fd = RNIL);
+
+  /**
+   * Allocate fragment record
+   */
+  int add_fragment(Uint32 tableId, Uint32 fragmentId);
+
+  /**
+   * Drop fragment record
+   */
+  void drop_fragment(Uint32 tableId, Uint32 fragmentId);
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
