@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -51,6 +58,11 @@ public:
     m_mgmsrv = m;
   }
   
+  ~Ndb_mgmd_event_service()
+  {
+    stop_sessions();
+  }
+
   void add_listener(const Event_listener&);
   void check_listeners();
   void update_max_log_level(const LogLevel&);
@@ -75,27 +87,6 @@ public:
 class MgmtSrvr : private ConfigSubscriber, public trp_client {
 
 public:
-  // some compilers need all of this
-  class Allocated_resources;
-  friend class Allocated_resources;
-  class Allocated_resources {
-  public:
-    Allocated_resources(class MgmtSrvr &m);
-    ~Allocated_resources();
-    // methods to reserve/allocate resources which
-    // will be freed when running destructor
-    void reserve_node(NodeId id, NDB_TICKS timeout);
-    bool is_timed_out(NDB_TICKS tick);
-    bool is_reserved(NodeId nodeId) { return m_reserved_nodes.get(nodeId); }
-    bool is_reserved(NodeBitmask mask) { return !mask.bitAND(m_reserved_nodes).isclear(); }
-    bool isclear() { return m_reserved_nodes.isclear(); }
-    NodeId get_nodeid() const;
-  private:
-    MgmtSrvr &m_mgmsrv;
-    NodeBitmask m_reserved_nodes;
-    NDB_TICKS m_alloc_timeout;
-  };
-
   /**
    *   This enum specifies the different signal loggig modes possible to set 
    *   with the setSignalLoggingMode method.
@@ -113,13 +104,14 @@ public:
     int interactive;
     const char* config_filename;
     int mycnf;
+    const char* cluster_config_suffix;
     int config_cache;
     const char* bind_address;
     int no_nodeid_checks;
     int print_full_config;
     const char* configdir;
     int verbose;
-    MgmtOpts() : configdir(MYSQLCLUSTERDIR) {};
+    MgmtOpts() : configdir(MYSQLCLUSTERDIR) {}
     int reload;
     int initial;
     NodeBitmask nowait_nodes;
@@ -129,7 +121,7 @@ public:
   MgmtSrvr(const MgmtSrvr&); // Not implemented
   MgmtSrvr(const MgmtOpts&);
 
-  ~MgmtSrvr();
+  ~MgmtSrvr() override;
 
 private:
   /* Function used from 'init' */
@@ -154,7 +146,7 @@ private:
 
 public:
 
-  NodeId getOwnNodeId() const {return _ownNodeId;};
+  NodeId getOwnNodeId() const {return _ownNodeId;}
 
   /**
    * Get status on a node.
@@ -163,15 +155,18 @@ public:
    * Copy the string if you are not going to use it immediately.
    */
   int status(int nodeId,
-	     ndb_mgm_node_status * status,
-	     Uint32 * version,
-	     Uint32 * mysql_version,
-	     Uint32 * phase,
-	     bool * systemShutdown,
-	     Uint32 * dynamicId,
-	     Uint32 * nodeGroup,
-	     Uint32 * connectCount,
-	     const char **address);
+             ndb_mgm_node_status * status,
+             Uint32 * version,
+             Uint32 * mysql_version,
+             Uint32 * phase,
+             bool * systemShutdown,
+             Uint32 * dynamicId,
+             Uint32 * nodeGroup,
+             Uint32 * connectCount,
+             const char **address,
+             char *addr_buf,
+             size_t addr_buf_size,
+             bool* is_single_user);
 
   /**
    *   Stop a list of nodes
@@ -198,11 +193,11 @@ public:
   int exitSingleUser(int * cnt = 0, bool abort = false);
 
   /**
-   *   Start DB process.
-   *   @param   processId: Id of the DB process to start
+   *   Start DB process by sending START_ORD to it.
+   *   @param   processId  Id of the DB process to start
    *   @return 0 if succeeded, otherwise: as stated above, plus:
    */
- int start(int processId);
+ int sendSTART_ORD(int processId);
 
   /**
    *   Restart a list of nodes
@@ -210,19 +205,24 @@ public:
   int restartNodes(const Vector<NodeId> &node_ids,
                    int *stopCount, bool nostart,
                    bool initialStart, bool abort, bool force,
-                   int *stopSelf);
+                   int *stopSelf,
+                   unsigned int num_secs_to_wait_for_node = 120);
 
   /**
    *   Restart all DB nodes
    */
   int restartDB(bool nostart, bool initialStart, 
                 bool abort = false,
-                int * stopCount = 0);
+                int * stopCount = 0,
+                unsigned int num_secs_to_wait_for_node = 120);
   
   /**
    * Backup functionallity
    */
-  int startBackup(Uint32& backupId, int waitCompleted= 2, Uint32 input_backupId= 0, Uint32 backuppoint= 0);
+  int startBackup(Uint32& backupId, int waitCompleted= 2,
+                  Uint32 input_backupId= 0, Uint32 backuppoint= 0,
+                  const char* encryption_password= nullptr,
+                  Uint32 password_length= 0);
   int abortBackup(Uint32 backupId);
   int performBackup(Uint32* backupId);
 
@@ -241,12 +241,12 @@ public:
 
   /**
    *   Insert an error in a DB process.
-   *   @param   processId: Id of the DB process
-   *   @param   errorNo: The error number. > 0.
+   *   @param   processId  Id of the DB process
+   *   @param   errorNo    The error number. > 0.
    *   @return  0 if succeeded, otherwise: as stated above, plus:
    *            INVALID_ERROR_NUMBER
    */
-  int insertError(int processId, int errorNo);
+  int insertError(int processId, int errorNo, Uint32 * extra = 0);
 
 
 
@@ -301,6 +301,7 @@ public:
   /**
    *   Dump State 
    */
+  int dumpStateSelf(const Uint32 args[], Uint32 no);
   int dumpState(int processId, const Uint32 args[], Uint32 argNo);
   int dumpState(int processId, const char* args);
 
@@ -311,50 +312,57 @@ public:
    *   @return false if none found
    */
   bool getNextNodeId(NodeId * _nodeId, enum ndb_mgm_node_type type) const ;
-  bool alloc_node_id(NodeId * _nodeId, enum ndb_mgm_node_type type,
-		     const struct sockaddr *client_addr,
-                     SOCKET_SIZE_TYPE *client_addr_len,
-		     int &error_code, BaseString &error_string,
-                     int log_event = 1,
-		     int timeout_s = 20);
 
   bool change_config(Config& new_config, BaseString& msg);
 
   /**
    *   Get error text
-   * 
-   *   @param   errorCode: Error code to get a match error text for.
+   *
+   *   @param   errorCode  Error code to get a match error text for.
    *   @return  The error text.
    */
   const char* getErrorText(int errorCode, char *buf, int buf_sz);
 
 private:
-  void config_changed(NodeId, const Config*);
+  void config_changed(NodeId, const Config*) override;
   void setClusterLog(const Config* conf);
+  void configure_eventlogger(const BaseString& logdestination) const;
+  /**
+   * Make cluster logging asynchronous/synchronous
+   * when g_eventLogger is set up the next time.
+   * @param async_cluster_logging true for async logging,
+   * false for sync logging
+   */
+  void set_async_cluster_logging(bool async_cluster_logging);
 public:
 
   /**
    * Returns the port number where MgmApiService is started
    * @return port number.
    */
-  int getPort() const { return m_port; };
+  int getPort() const { return m_port; }
 
   int setDbParameter(int node, int parameter, const char * value, BaseString&);
   int setConnectionDbParameter(int node1, int node2, int param, int value,
 			       BaseString& msg);
+
+  struct DynPortSpec {
+   int node; /* The node which should use below port */
+   int port; /* The port to use */
+  };
+  bool setDynamicPorts(int node, DynPortSpec ports[],
+                       unsigned num_ports, BaseString &msg);
   int getConnectionDbParameter(int node1, int node2, int param,
 			       int *value, BaseString& msg);
 
-  bool transporter_connect(NDB_SOCKET_TYPE sockfd, BaseString& errormsg);
+  bool transporter_connect(NDB_SOCKET_TYPE sockfd,
+                           BaseString& errormsg,
+                           bool& close_with_reset);
 
-  const char *get_connect_address(Uint32 node_id);
-  void get_connected_nodes(NodeBitmask &connected_nodes) const;
   SocketServer *get_socket_server() { return &m_socket_server; }
 
-  void updateStatus();
-
   int createNodegroup(int *nodes, int count, int *ng);
-  int dropNodegroup(int ng);
+  int dropNodegroup(unsigned ng);
 
   int startSchemaTrans(SignalSender& ss, NodeId & out_nodeId,
                        Uint32 transId, Uint32 & out_transKey);
@@ -367,14 +375,25 @@ private:
   void status_api(int nodeId,
                   ndb_mgm_node_status& node_status,
                   Uint32& version, Uint32& mysql_version,
-                  const char **address);
+                  const char **address,
+                  char *addr_buf,
+                  size_t addr_buf_size,
+                  bool& is_single_user);
   void status_mgmd(NodeId node_id,
                    ndb_mgm_node_status& node_status,
                    Uint32& version, Uint32& mysql_version,
-                   const char **address);
+                   const char **address,
+                   char *addr_buf,
+                   size_t addr_buf_size,
+                   bool& is_single_user);
 
-  int sendVersionReq(int processId, Uint32 &version,
-                     Uint32& mysql_version, const char **address);
+  int sendVersionReq(int processId,
+                     Uint32 &version,
+                     Uint32& mysql_version,
+                     const char **address,
+                     char *addr_buf,
+                     size_t addr_buf_size,
+                     bool& is_single_user);
 
   int sendStopMgmd(NodeId nodeId,
                    bool abort,
@@ -398,35 +417,29 @@ private:
 		   bool nostart,
 		   bool initialStart,
                    int *stopSelf);
- 
+
   /**
    *   Check if it is possible to send a signal to a (DB) process
    *
-   *   @param   processId: Id of the process to send to
+   *   @param   nodeId  Id of the node to send to
    *   @return  0 OK, 1 process dead, 2 API or MGMT process, 3 not configured
    */
   int okToSendTo(NodeId nodeId, bool unCond = false);
 
-  /**
-   *   Get block number for a block
-   *
-   *   @param   blockName: Block to get number for
-   *   @return  -1 if block not found, otherwise block number
-   */
-  int getBlockNumber(const BaseString &blockName);
-
-  int alloc_node_id_req(NodeId free_node_id,
-                        enum ndb_mgm_node_type type,
-                        Uint32 timeout_ms);
-
   bool is_any_node_starting(void);
   bool is_any_node_stopping(void);
+  bool is_any_node_alive(void);
+  bool is_any_node_in_started_state(void);
   bool is_cluster_single_user(void);
+  bool are_all_nodes_in_cmvmi_state(void);
+  bool isTimeUp(const NDB_TICKS startTime,
+                const Uint64 delay,
+                const Uint64 startInterval);
 
   //**************************************************************************
 
   const MgmtOpts& m_opts;
-  int _blockNumber;
+  BlockNumber _blockNumber;
   NodeId _ownNodeId;
   Uint32 m_port;
   SocketServer m_socket_server;
@@ -434,28 +447,31 @@ private:
   NdbMutex* m_local_config_mutex;
   const Config* m_local_config;
 
-  NdbMutex *m_node_id_mutex;
-
   BlockReference _ownReference;
 
   class ConfigManager* m_config_manager;
 
   bool m_need_restart;
 
-  NodeBitmask m_reserved_nodes;
-  struct in_addr m_connect_address[MAX_NODES];
+  struct in6_addr m_connect_address[MAX_NODES];
+  const char *get_connect_address(NodeId node_id,
+                                  char *addr_buf,
+                                  size_t addr_buf_size);
+  void clear_connect_address_cache(NodeId nodeid);
 
   /**
    * trp_client interface
    */
-  virtual void trp_deliver_signal(const NdbApiSignal* signal,
-                                  const struct LinearSectionPtr ptr[3]);
+  void trp_deliver_signal(const NdbApiSignal* signal,
+                          const struct LinearSectionPtr ptr[3]) override;
   virtual void trp_node_status(Uint32 nodeId, Uint32 event);
   
   /**
    * An event from <i>nodeId</i> has arrived
    */
-  void eventReport(const Uint32 * theData, Uint32 len);
+  void eventReport(const Uint32 * theSignalData,
+                   Uint32 len,
+                   const Uint32 * theData);
  
   class TransporterFacade * theFacade;
 
@@ -486,11 +502,16 @@ private:
 public:
   /* Get copy of configuration packed with base64 */
   bool get_packed_config(ndb_mgm_node_type nodetype,
-                         BaseString& buf64, BaseString& error);
+                         BaseString& buf64,
+                         BaseString& error,
+                         bool v2,
+                         Uint32 node_id);
 
   /* Get copy of configuration packed with base64 from node nodeid */
   bool get_packed_config_from_node(NodeId nodeid,
-                         BaseString& buf64, BaseString& error);
+                         BaseString& buf64,
+                         BaseString& error,
+                         bool v2);
 
   void print_config(const char* section_filter = NULL,
                     NodeId nodeid_filter = 0,
@@ -502,21 +523,95 @@ public:
 
   void show_variables(NdbOut& out = ndbout);
 
-  struct nodeid_and_host
+private:
+  class NodeIdReservations {
+    struct Reservation {
+      NDB_TICKS m_start;
+      unsigned m_timeout; // Milliseconds
+    };
+    Reservation m_reservations[MAX_NODES];
+  public:
+    NodeIdReservations();
+    void check_array(NodeId n) const;
+    bool get(NodeId n) const;
+    void set(NodeId n, unsigned timeout);
+    void clear(NodeId n);
+    BaseString pretty_str() const;
+    bool has_timedout(NodeId n, NDB_TICKS now) const;
+  } m_reserved_nodes;
+  NdbMutex* m_reserved_nodes_mutex;
+
+  void release_local_nodeid_reservation(NodeId nodeid);
+  struct PossibleNode
   {
     unsigned id;
     BaseString host;
+    bool exact_match;
   };
-  int find_node_type(unsigned node_id, enum ndb_mgm_node_type type,
-                     const struct sockaddr *client_addr,
-                     NodeBitmask &nodes,
-                     NodeBitmask &exact_nodes,
-                     Vector<nodeid_and_host> &nodes_info,
-                     int &error_code, BaseString &error_string);
-  int try_alloc(unsigned id,  const char *, enum ndb_mgm_node_type type,
-                const struct sockaddr *client_addr, Uint32 timeout_ms);
+  int alloc_node_id_req(NodeId free_node_id,
+                        ndb_mgm_node_type type,
+                        Uint32 timeout_ms);
+  int try_alloc(NodeId id,
+                ndb_mgm_node_type type,
+                Uint32 timeout_ms,
+                int& error_code,
+                BaseString& error_string);
+  int try_alloc_from_list(NodeId& nodeid,
+                          ndb_mgm_node_type type,
+                          Uint32 timeout_ms,
+                          const Vector<PossibleNode>& nodes_info,
+                          int& error_code,
+                          BaseString& error_string);
+  struct ConfigNode
+  {
+    unsigned nodeid;
+    BaseString hostname;
+  };
+  bool build_node_list_from_config(NodeId node_id,
+                                   ndb_mgm_node_type type,
+                                   Vector<ConfigNode>& config_nodes,
+                                   int& error_code,
+                                   BaseString& error_string) const;
+  int find_node_type(NodeId nodeid,
+                     ndb_mgm_node_type type,
+                     const sockaddr_in6* client_addr,
+                     const Vector<ConfigNode>& config_nodes,
+                     Vector<PossibleNode>& nodes,
+                     int& error_code, BaseString& error_string);
+  bool alloc_node_id_impl(NodeId& nodeid,
+                          ndb_mgm_node_type type,
+                          const sockaddr_in6* client_addr,
+                          int& error_code, BaseString& error_string,
+                          Uint32 timeout_s = 20);
+public:
+  /*
+    Nodeid allocation
+    - MGM will never allocate a nodeid since either there is
+      only one(and thus it's nodeid is well known by looking
+      in config) or when running with two or more, they will
+      have nodeid specified on command line when starting.
+    - API nodes will always allocate a nodeid in the data nodes
+      since those have transporter connections with hearbeat
+      mechanism and can thus detect when the allocation should
+      be cleared.
+    - NDB nodes will prefer to allocate a nodeid in the data nodes
+      since the allocations will then be known by all connected
+      nodes(as well as hearbeat). But in order to startup a
+      nodeid can also be reserved locally in the mgmtsrvr node
+      only, such a reservations is cleared as soon as the
+      data node has connected via transporter to the mgmtsrvr.
+   */
+  bool alloc_node_id(NodeId& nodeid,
+                     ndb_mgm_node_type type,
+                     const sockaddr_in6* client_addr,
+		     int& error_code, BaseString& error_string,
+                     bool log_event = true,
+		     Uint32 timeout_s = 20);
 
+private:
   BaseString m_version_string;
+  bool m_async_cluster_logging;
+public:
   const char* get_version_string(void) const {
     return m_version_string.c_str();
   }
