@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -19,6 +26,8 @@
 #ifndef NdbQueryBuilderImpl_H
 #define NdbQueryBuilderImpl_H
 
+#include <cstring>
+
 /* Query-related error codes. */
 #define QRY_REQ_ARG_IS_NULL 4800
 #define QRY_TOO_FEW_KEY_VALUES 4801
@@ -27,8 +36,7 @@
 #define QRY_CHAR_OPERAND_TRUNCATED 4804
 #define QRY_NUM_OPERAND_RANGE 4805
 #define QRY_MULTIPLE_PARENTS 4806
-#define QRY_UNKONWN_PARENT 4807
-#define QRY_UNKNOWN_COLUMN 4808
+#define QRY_UNKNOWN_PARENT 4807
 #define QRY_UNRELATED_INDEX 4809
 #define QRY_WRONG_INDEX_TYPE 4810
 #define QRY_OPERAND_ALREADY_BOUND 4811
@@ -44,8 +52,12 @@
 #define QRY_CHAR_PARAMETER_TRUNCATED 4823
 #define QRY_MULTIPLE_SCAN_SORTED 4824
 #define QRY_BATCH_SIZE_TOO_SMALL 4825
+#define QRY_EMPTY_PROJECTION 4826
+#define QRY_OJ_NOT_SUPPORTED 4827
+//#define QRY_NEST_NOT_SPECIFIED 4828  <<== DEPRECATED
+#define QRY_NEST_NOT_SUPPORTED 4829
+
 
-#ifdef __cplusplus
 #include <Vector.hpp>
 #include <Bitmask.hpp>
 #include "NdbQueryBuilder.hpp"
@@ -53,6 +65,7 @@
 #include "ndb_limits.h"
 
 // Forward declared
+class Ndb;
 class NdbTableImpl;
 class NdbIndexImpl;
 class NdbColumnImpl;
@@ -93,9 +106,9 @@ public:
 //#define TEST_Uint32Buffer
 
 #if defined(TEST_Uint32Buffer)
-  STATIC_CONST(initSize = 1);  // Small size to force test of buffer expand.
+  static constexpr Uint32 initSize = 1;  // Small size to force test of buffer expand.
 #else
-  STATIC_CONST(initSize = 32); // Initial buffer size, extend on demand but probably sufficent
+  static constexpr Uint32 initSize = 32; // Initial buffer size, extend on demand but probably sufficent
 #endif
 
   explicit Uint32Buffer():
@@ -211,7 +224,7 @@ public:
         memcpy(start, src, len);
         m_bytesLeft = (m_bytesLeft - len) % sizeof(Uint32);
         // Make sure that any trailing bytes in the last word are zero.
-        bzero(start + len, m_bytesLeft);
+        std::memset(start + len, 0, m_bytesLeft);
       }
     }
   }
@@ -268,17 +281,26 @@ public:
   explicit NdbQueryOptionsImpl()
   : m_matchType(NdbQueryOptions::MatchAll),
     m_scanOrder(NdbQueryOptions::ScanOrdering_void),
-    m_parent(NULL),
-    m_interpretedCode(NULL)
-  {};
+    m_parent(nullptr),
+    m_firstUpper(nullptr),
+    m_firstInner(nullptr),
+    m_interpretedCode(nullptr),
+    m_parameters(0)
+  {}
   NdbQueryOptionsImpl(const NdbQueryOptionsImpl&);
   ~NdbQueryOptionsImpl();
 
+  NdbQueryOptions::ScanOrdering getOrdering() const
+  { return m_scanOrder; }
+
 private:
   NdbQueryOptions::MatchType     m_matchType;
   NdbQueryOptions::ScanOrdering  m_scanOrder;
   NdbQueryOperationDefImpl*      m_parent;
+  NdbQueryOperationDefImpl*      m_firstUpper;   //First in upper nest
+  NdbQueryOperationDefImpl*      m_firstInner;   //First in this (inner-)nest
   const NdbInterpretedCode*      m_interpretedCode;
+  Vector<const NdbQueryOperandImpl*> m_parameters;
 
   /**
    * Assign NdbInterpretedCode by taking a deep copy of 'src'
@@ -313,27 +335,55 @@ public:
   Uint32 getNoOfParentOperations() const
   { return (m_parent) ? 1 : 0; }
 
-  NdbQueryOperationDefImpl& getParentOperation(Uint32 i) const
+  const NdbQueryOperationDefImpl& getParentOperation(Uint32 i
+                                                     [[maybe_unused]]) const
   { assert(i==0 && m_parent!=NULL);
     return *m_parent;
   }
 
-  NdbQueryOperationDefImpl* getParentOperation() const
+  const NdbQueryOperationDefImpl* getParentOperation() const
   { return m_parent;
   }
 
   Uint32 getNoOfChildOperations() const
   { return m_children.size(); }
 
-  NdbQueryOperationDefImpl& getChildOperation(Uint32 i) const
+  const NdbQueryOperationDefImpl& getChildOperation(Uint32 i) const
   { return *m_children[i]; }
 
+  const NdbQueryOperationDefImpl* getFirstInner() const
+  { return m_firstInner; }
+
+  const NdbQueryOperationDefImpl* getFirstInEmbeddingNest() const
+  {
+    assert(m_firstInner == nullptr || m_firstUpper == nullptr);
+    if (m_firstInner != nullptr)
+      return m_firstInner;
+    else if (m_firstUpper != nullptr)
+      return m_firstUpper;
+    else
+      return nullptr;
+  }
+
   const NdbTableImpl& getTable() const
   { return m_table; }
 
   const char* getName() const
   { return m_ident; }
 
+  // Does an ancestor specify a MatchType requiring only a 'firstMatch'?
+  // Both 'MatchFirst' and 'MatchNullOnly' are a firstMatch type as it
+  // allows us to conclude as soon as a single qualifying row has been found.
+  bool hasFirstMatchAncestor() const
+  {
+    if (m_parent == nullptr)
+      return false;
+    if (m_parent->getMatchType() &
+	(NdbQueryOptions::MatchFirst | NdbQueryOptions::MatchNullOnly))
+      return true;
+    return m_parent->hasFirstMatchAncestor();
+  }
+
   enum NdbQueryOptions::MatchType getMatchType() const
   { return m_options.m_matchType; }
 
@@ -343,11 +393,8 @@ public:
   const NdbInterpretedCode* getInterpretedCode() const
   { return m_options.m_interpretedCode; }
 
-  Uint32 assignQueryOperationId(Uint32& nodeId)
-  { if (getType()==NdbQueryOperationDef::UniqueIndexAccess) nodeId++;
-    m_id = nodeId++;
-    return m_id;
-  }
+  const Vector<const NdbQueryOperandImpl*>& getInterpretedParams() const
+  { return m_options.m_parameters; }
 
   // Establish a linked parent <-> child relationship with this operation
   int linkWithParent(NdbQueryOperationDefImpl* parentOp);
@@ -385,7 +432,16 @@ public:
   virtual const IndexBound* getBounds() const
   { return NULL; } 
 
-  // Return 'true' is query type is a multi-row scan
+  /** 
+   * True if this is a prunable scan and there are NdbQueryParamOperands in the
+   * distribution key.
+   */
+  virtual bool hasParamInPruneKey() const
+  {
+    return false;
+  }
+
+  // Return 'true' if query type is a multi-row scan
   virtual bool isScanOperation() const = 0;
 
   virtual const NdbQueryOperationDef& getInterface() const = 0; 
@@ -394,7 +450,8 @@ public:
    * the struct QueryNode type.
    * @return Possible error code.
    */
-  virtual int serializeOperation(Uint32Buffer& serializedTree) = 0;
+  virtual int serializeOperation(const Ndb *ndb,
+                                 Uint32Buffer& serializedTree) = 0;
 
   /** Find the projection that should be sent to the SPJ block. This should
    * contain the attributes needed to instantiate all child operations.
@@ -403,10 +460,10 @@ public:
     return m_spjProjection;
   }
 
-  virtual int checkPrunable(const Uint32Buffer& keyInfo,
-                            Uint32  shortestBound,
-                            bool&   isPruned,
-			    Uint32& hashValue) const {
+  virtual int checkPrunable(const Uint32Buffer& /*keyInfo*/,
+                            Uint32 /*shortestBound*/, bool& isPruned,
+                            Uint32& /*hashValue*/) const
+  {
     isPruned = false;
     return 0;
   }
@@ -417,20 +474,27 @@ protected:
   explicit NdbQueryOperationDefImpl (const NdbTableImpl& table,
                                      const NdbQueryOptionsImpl& options,
                                      const char* ident,
-                                     Uint32 ix,
+                                     Uint32 opNo,
+                                     Uint32 internalOpNo,
                                      int& error);
 public:
   // Get the ordinal position of this operation within the query def.
-  Uint32 getQueryOperationIx() const
-  { return m_ix; }
+  Uint32 getOpNo() const
+  { return m_opNo; }
 
   // Get id of node as known inside queryTree
-  Uint32 getQueryOperationId() const
-  { return m_id; }
+  Uint32 getInternalOpNo() const
+  { return m_internalOpNo; }
 
   // Get type of query operation
   virtual NdbQueryOperationDef::Type getType() const = 0;
 
+  /**
+   * Used for telling if parent at depth n has more siblings. (In that case
+   * we need to draw a horisontal line leading to that sibling.)
+   */
+  typedef Bitmask<(NDB_SPJ_MAX_TREE_NODES+31)/32> SiblingMask;
+
   /** Print query tree graph to trace file (using recursion).
    * @param depth Number of ancestor nodes that this node has.
    * @param hasMoreSiblingsMask The n'th bit should be set if the n'th ancestor
@@ -438,7 +502,7 @@ public:
    */
   void printTree(
            Uint32 depth, 
-           Bitmask<(NDB_SPJ_MAX_TREE_NODES+31)/32> hasMoreSiblingsMask) const;
+           SiblingMask hasMoreSiblingsMask) const;
 
 protected:
   // QueryTree building:
@@ -448,6 +512,8 @@ protected:
   // Append list of columns required by SPJ to instantiate child operations.
   Uint32 appendChildProjection(Uint32Buffer& serializedDef) const;
 
+  Uint32 appendParamConstructor(Uint32Buffer& serializedDef) const;
+
 protected:
   /** True if enclosing query has been prepared.*/
   bool m_isPrepared;
@@ -474,9 +540,9 @@ private:
 private:
   const NdbTableImpl& m_table;
   const char* const m_ident; // Optional name specified by aplication
-  const Uint32 m_ix;         // Index of this operation within operation array
-  Uint32       m_id;         // Operation id when materialized into queryTree.
-                             // If op has index, index id is 'm_id-1'.
+  const Uint32 m_opNo;       // Index of this operation within operation array
+  const Uint32 m_internalOpNo;// Operation id when materialized into queryTree.
+                          // If op has index, index opNo is 'm_internalOpNo-1'.
 
   // Optional (or default) options specified when building query:
   // - Scan order which may specify ascending or descending scan order
@@ -488,6 +554,13 @@ private:
   NdbQueryOperationDefImpl* m_parent;
   Vector<NdbQueryOperationDefImpl*> m_children;
 
+  // The (optional) first table in the upper- or the inner nest of
+  // this table. Only set if the entire inner-nest is not contained
+  // within the tree branch starting with the first inner, or
+  // the first upper op-node.
+  const NdbQueryOperationDefImpl* const m_firstUpper;
+  const NdbQueryOperationDefImpl* const m_firstInner;
+
   // Params required by this operation
   Vector<const NdbParamOperandImpl*> m_params;
 
@@ -504,21 +577,23 @@ public:
                            const NdbTableImpl& table,
                            const NdbQueryOptionsImpl& options,
                            const char* ident,
-                           Uint32      ix,
+                           Uint32      opNo,
+                           Uint32      internalOpNo,
                            int& error);
 
-  virtual bool isScanOperation() const
+  bool isScanOperation() const override
   { return true; }
 
 protected:
-  int serialize(Uint32Buffer& serializedDef,
+  int serialize(const Ndb *ndb,
+                Uint32Buffer& serializedDef,
                 const NdbTableImpl& tableOrIndex);
 
-  // Append pattern for creating complete range bounds to serialized code 
-  virtual Uint32 appendBoundPattern(Uint32Buffer& serializedDef) const
+  // Append pattern for creating complete range bounds to serialized code
+  virtual Uint32 appendBoundPattern(Uint32Buffer& /*serializedDef*/) const
   { return 0; }
 
-  virtual Uint32 appendPrunePattern(Uint32Buffer& serializedDef) const
+  virtual Uint32 appendPrunePattern(Uint32Buffer& /*serializedDef*/)
   { return 0; }
 
 }; // class NdbQueryScanOperationDefImpl
@@ -529,30 +604,36 @@ class NdbQueryIndexScanOperationDefImpl : public NdbQueryScanOperationDefImpl
   friend class NdbQueryBuilder;  // Allow privat access from builder interface
 
 public:
-  virtual const NdbIndexImpl* getIndex() const
+  const NdbIndexImpl* getIndex() const override
   { return &m_index; }
 
-  virtual int serializeOperation(Uint32Buffer& serializedDef);
+  int serializeOperation(const Ndb *ndb,
+                                 Uint32Buffer& serializedDef) override;
 
-  virtual const NdbQueryIndexScanOperationDef& getInterface() const
+  const NdbQueryIndexScanOperationDef& getInterface() const override
   { return m_interface; }
 
-  virtual NdbQueryOperationDef::Type getType() const
+  NdbQueryOperationDef::Type getType() const override
   { return NdbQueryOperationDef::OrderedIndexScan; }
 
-  virtual int checkPrunable(const Uint32Buffer& keyInfo,
+  int checkPrunable(const Uint32Buffer& keyInfo,
                             Uint32  shortestBound,
                             bool&   isPruned,
-                            Uint32& hashValue) const;
+                            Uint32& hashValue) const override;
 
-  virtual const IndexBound* getBounds() const
+  const IndexBound* getBounds() const override
   { return &m_bound; } 
 
+  bool hasParamInPruneKey() const override
+  {
+    return m_paramInPruneKey;
+  }
+
 protected:
   // Append pattern for creating complete range bounds to serialized code 
-  virtual Uint32 appendBoundPattern(Uint32Buffer& serializedDef) const;
+  Uint32 appendBoundPattern(Uint32Buffer& serializedDef) const override;
 
-  virtual Uint32 appendPrunePattern(Uint32Buffer& serializedDef) const;
+  Uint32 appendPrunePattern(Uint32Buffer& serializedDef) override;
 
 private:
 
@@ -562,7 +643,8 @@ private:
                            const NdbQueryIndexBound* bound,
                            const NdbQueryOptionsImpl& options,
                            const char* ident,
-                           Uint32      ix,
+                           Uint32      opNo,
+                           Uint32      internalOpNo,
                            int& error);
 
   // Append pattern for creating a single bound value to serialized code 
@@ -577,6 +659,12 @@ private:
 
   /** True if there is a set of bounds.*/
   IndexBound m_bound;
+
+  /** 
+   * True if scan is prunable and there are NdbQueryParamOperands in the 
+   * distribution key.
+   */
+  bool m_paramInPruneKey;
 }; // class NdbQueryIndexScanOperationDefImpl
 
 
@@ -585,7 +673,8 @@ class NdbQueryDefImpl
   friend class NdbQueryDef;
 
 public:
-  explicit NdbQueryDefImpl(const Vector<NdbQueryOperationDefImpl*>& operations,
+  explicit NdbQueryDefImpl(const Ndb *ndb,
+                           const Vector<NdbQueryOperationDefImpl*>& operations,
                            const Vector<NdbQueryOperandImpl*>& operands,
                            int& error);
   ~NdbQueryDefImpl();
@@ -635,7 +724,7 @@ public:
   ~NdbQueryBuilderImpl();
   explicit NdbQueryBuilderImpl();
 
-  const NdbQueryDefImpl* prepare();
+  const NdbQueryDefImpl* prepare(const Ndb *ndb);
 
   const NdbError& getNdbError() const;
 
@@ -656,18 +745,27 @@ private:
   /**
    * Take ownership of specified object: From now on it is the
    * responsibility of this NdbQueryBuilderImpl to manage the
-   * lifetime of the object. If takeOwnership() fails, the 
+   * lifetime of the object. If takeOwnership() fails, the
    * specified object is deleted before it returns.
-   * @param[in] operand to take ownership for (may be NULL).
+   *
+   * @param[in] operand operand to take ownership for (may be NULL).
    * @return 0 if ok, else there has been an 'Err_MemoryAlloc'
    */
-  int takeOwnership(NdbQueryOperandImpl*);
+  int takeOwnership(NdbQueryOperandImpl* operand);
   int takeOwnership(NdbQueryOperationDefImpl*);
 
   bool contains(const NdbQueryOperationDefImpl*);
 
+  // Get interal operation number of the next operation.
+  Uint32 getNextInternalOpNo() const
+  { 
+    return m_operations.size() == 0 ? 0 :
+      m_operations[m_operations.size()-1]->getInternalOpNo()+1;
+  }
+
   NdbQueryBuilder m_interface;
-  NdbError m_error;
+  // Allow update error from const methods
+  mutable NdbError m_error;
 
   Vector<NdbQueryOperationDefImpl*> m_operations;
   Vector<NdbQueryOperandImpl*> m_operands;
@@ -700,7 +798,7 @@ public:
   { return m_column; }
 
   virtual int bindOperand(const NdbColumnImpl& column,
-                          NdbQueryOperationDefImpl& operation)
+                          NdbQueryOperationDefImpl& /*operation*/)
   { if (m_column  && m_column != &column)
       // Already bounded to a different column
       return QRY_OPERAND_ALREADY_BOUND;
@@ -717,7 +815,7 @@ protected:
   friend NdbQueryBuilderImpl::~NdbQueryBuilderImpl();
   friend NdbQueryDefImpl::~NdbQueryDefImpl();
 
-  virtual ~NdbQueryOperandImpl(){};
+  virtual ~NdbQueryOperandImpl(){}
 
   NdbQueryOperandImpl(Kind kind)
     : m_column(0),
@@ -750,11 +848,11 @@ public:
   const NdbColumnImpl& getParentColumn() const
   { return *m_parentOperation.getSPJProjection()[m_parentColumnIx]; }
 
-  virtual NdbQueryOperand& getInterface()
+  NdbQueryOperand& getInterface() override
   { return m_interface; }
 
-  virtual int bindOperand(const NdbColumnImpl& column,
-                          NdbQueryOperationDefImpl& operation);
+  int bindOperand(const NdbColumnImpl& column,
+                  NdbQueryOperationDefImpl& operation) override;
 
 private:
   NdbLinkedOperandImpl (NdbQueryOperationDefImpl& parent, 
@@ -782,11 +880,11 @@ public:
   Uint32 getParamIx() const
   { return m_paramIx; }
 
-  virtual NdbQueryOperand& getInterface()
+  NdbQueryOperand& getInterface() override
   { return m_interface; }
 
-  virtual int bindOperand(const NdbColumnImpl& column,
-                          NdbQueryOperationDefImpl& operation);
+  int bindOperand(const NdbColumnImpl& column,
+                  NdbQueryOperationDefImpl& operation) override;
 
 private:
   NdbParamOperandImpl (const char* name, Uint32 paramIx)
@@ -811,11 +909,11 @@ public:
   const void* getAddr() const
   { return likely(m_converted.buffer==NULL) ? &m_converted.val : m_converted.buffer; }
 
-  virtual NdbQueryOperand& getInterface()
+  NdbQueryOperand& getInterface() override
   { return m_interface; }
 
-  virtual int bindOperand(const NdbColumnImpl& column,
-                          NdbQueryOperationDefImpl& operation);
+  int bindOperand(const NdbColumnImpl& column,
+                  NdbQueryOperationDefImpl& operation) override;
 
 protected:
   NdbConstOperandImpl ()
@@ -827,35 +925,35 @@ protected:
   #define UNDEFINED_CONVERSION	\
   { return QRY_OPERAND_HAS_WRONG_TYPE; }
 
-  virtual int convertUint8()  UNDEFINED_CONVERSION;
-  virtual int convertInt8()   UNDEFINED_CONVERSION;
-  virtual int convertUint16() UNDEFINED_CONVERSION;
-  virtual int convertInt16()  UNDEFINED_CONVERSION;
-  virtual int convertUint24() UNDEFINED_CONVERSION;
-  virtual int convertInt24()  UNDEFINED_CONVERSION;
-  virtual int convertUint32() UNDEFINED_CONVERSION;
-  virtual int convertInt32()  UNDEFINED_CONVERSION;
-  virtual int convertUint64() UNDEFINED_CONVERSION;
-  virtual int convertInt64()  UNDEFINED_CONVERSION;
-  virtual int convertFloat()  UNDEFINED_CONVERSION;
+  virtual int convertUint8()  UNDEFINED_CONVERSION
+  virtual int convertInt8()   UNDEFINED_CONVERSION
+  virtual int convertUint16() UNDEFINED_CONVERSION
+  virtual int convertInt16()  UNDEFINED_CONVERSION
+  virtual int convertUint24() UNDEFINED_CONVERSION
+  virtual int convertInt24()  UNDEFINED_CONVERSION
+  virtual int convertUint32() UNDEFINED_CONVERSION
+  virtual int convertInt32()  UNDEFINED_CONVERSION
+  virtual int convertUint64() UNDEFINED_CONVERSION
+  virtual int convertInt64()  UNDEFINED_CONVERSION
+  virtual int convertFloat()  UNDEFINED_CONVERSION
   virtual int convertDouble() UNDEFINED_CONVERSION
 
-  virtual int convertUDec()   UNDEFINED_CONVERSION;
-  virtual int convertDec()    UNDEFINED_CONVERSION;
+  virtual int convertUDec()   UNDEFINED_CONVERSION
+  virtual int convertDec()    UNDEFINED_CONVERSION
 
-  virtual int convertBit()    UNDEFINED_CONVERSION;
-  virtual int convertChar()   UNDEFINED_CONVERSION;
-  virtual int convertVChar()  UNDEFINED_CONVERSION;
-  virtual int convertLVChar() UNDEFINED_CONVERSION;
-  virtual int convertBin()    UNDEFINED_CONVERSION;
-  virtual int convertVBin()   UNDEFINED_CONVERSION;
-  virtual int convertLVBin()  UNDEFINED_CONVERSION;
+  virtual int convertBit()    UNDEFINED_CONVERSION
+  virtual int convertChar()   UNDEFINED_CONVERSION
+  virtual int convertVChar()  UNDEFINED_CONVERSION
+  virtual int convertLVChar() UNDEFINED_CONVERSION
+  virtual int convertBin()    UNDEFINED_CONVERSION
+  virtual int convertVBin()   UNDEFINED_CONVERSION
+  virtual int convertLVBin()  UNDEFINED_CONVERSION
 
-  virtual int convertDate()   UNDEFINED_CONVERSION;
-  virtual int convertDatetime() UNDEFINED_CONVERSION;
-  virtual int convertTime()   UNDEFINED_CONVERSION;
-  virtual int convertYear()   UNDEFINED_CONVERSION;
-  virtual int convertTimestamp() UNDEFINED_CONVERSION;
+  virtual int convertDate()   UNDEFINED_CONVERSION
+  virtual int convertDatetime() UNDEFINED_CONVERSION
+  virtual int convertTime()   UNDEFINED_CONVERSION
+  virtual int convertYear()   UNDEFINED_CONVERSION
+  virtual int convertTimestamp() UNDEFINED_CONVERSION
 
   virtual int convert2ColumnType();
 
@@ -864,10 +962,10 @@ protected:
     */
   class ConvertedValue {
   public:
-    ConvertedValue()  : len(0), buffer(NULL) {};
+    ConvertedValue()  : len(0), buffer(NULL) {}
     ~ConvertedValue() {
       if (buffer) delete[] ((char*)buffer);
-    };
+    }
 
     char* getCharBuffer(Uint32 size) {
       char* dst = val.shortChar;
@@ -879,7 +977,7 @@ protected:
       return dst;
     }
 
-    STATIC_CONST(maxShortChar = 32);
+    static constexpr Uint32 maxShortChar = 32;
 
     union
     {
@@ -907,5 +1005,4 @@ private:
 }; // class NdbConstOperandImpl
 
 
-#endif /* __cplusplus */
 #endif
