@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -48,10 +55,11 @@ Revision history:
 
  * *************************************************** */
 
+#include "util/require.h"
 #include <ndb_global.h>
+#include <cstring>
 #include <NdbApi.hpp>
 
-#include <NdbMain.h>
 #include <NdbThread.h>
 #include <NdbSleep.h>
 #include <NdbTick.h>
@@ -103,7 +111,7 @@ struct ThreadNdb {
 extern "C" void* flexHammerThread(void*);
 static int setAttrNames(void);
 static int setTableNames(void);
-static int readArguments(int, const char**);
+static int readArguments(int, char**);
 static int createTables(Ndb*);
 static int dropTables(Ndb*);
 static void sleepBeforeStartingTest(int seconds);
@@ -148,11 +156,11 @@ void
 resetThreads(ThreadNdb *threadArrayP) {
 
   for (int i = 0; i < tNoOfThreads ; i++)
-    {
-      threadArrayP[i].threadReady = 0;
-      threadArrayP[i].threadResult = 0;
-      threadArrayP[i].threadStart = stIdle;
-    }
+  {
+    threadArrayP[i].threadReady = 0;
+    threadArrayP[i].threadResult = 0;
+    threadArrayP[i].threadStart = stIdle;
+  }
 } // resetThreads
 
 void 
@@ -165,7 +173,9 @@ waitForThreads(ThreadNdb *threadArrayP)
     cont = 0;
     for (int i = 0; i < tNoOfThreads ; i++) {
       if (threadArrayP[i].threadReady == 0) {
+        // Found one thread not yet ready, continue waiting
 	cont = 1;
+        break;
       } // if
     } // for
   } // while
@@ -182,8 +192,7 @@ tellThreads(ThreadNdb* threadArrayP, const StartType what)
 
 static Ndb_cluster_connection *g_cluster_connection= 0;
  
-NDB_COMMAND(flexHammer, "flexHammer", "flexHammer", "flexHammer", 65535)
-//main(int argc, const char** argv)
+int main(int argc, char** argv)
 {
   ndb_init();
   ThreadNdb* pThreads = NULL; // Pointer to thread data array
@@ -217,8 +226,11 @@ NDB_COMMAND(flexHammer, "flexHammer", "flexHammer", "flexHammer", 65535)
 
   // Create thread data array
   pThreads = new ThreadNdb[tNoOfThreads];
-  // NdbThread_SetConcurrencyLevel(tNoOfThreads + 2);
-
+  if (pThreads == nullptr)
+  {
+    ndbout << "Failed to allocate pThreads" << endl;
+    return NDBT_ProgramExit(NDBT_FAILED);
+  }
   // Create and init Ndb object
   Ndb_cluster_connection con;
   if(con.connect(12, 5, 1) != 0)
@@ -352,7 +364,7 @@ flexHammerThread(void* pArg)
   int tThreadResult = 0;
   MyOpType tMyOpType = otLast;
   int pkValue = 0;
-  int readValue[MAXATTR][MAXATTRSIZE]; bzero(readValue, sizeof(readValue));
+  int readValue[MAXATTR][MAXATTRSIZE]; std::memset(readValue, 0, sizeof(readValue));
   int attrValue[MAXATTRSIZE];
   NdbRecAttr* tTmp = NULL;
   int tNoOfAttempts = 0;
@@ -423,7 +435,7 @@ flexHammerThread(void* pArg)
 	  tMyOpType = otDelete;
 	  break;
 	default:
-	  assert(false);
+	  require(false);
 	  break;
 	} // switch
 	    
@@ -481,7 +493,7 @@ flexHammerThread(void* pArg)
 	    check = pMyOperation[count_tables]->deleteTuple();
 	    break;
 	  default:
-	    assert(false);
+	    require(false);
 	    break;
 	  } // switch
 	  if (check == -1) {
@@ -524,7 +536,7 @@ flexHammerThread(void* pArg)
 	  case otDelete:			// Delete Case
 	    break;
 	  default:
-	    assert(false);
+	    require(false);
 	    break;
 	  } // switch
 	  if (check == -1 && tTmp == NULL && tMyOpType != otDelete) {
@@ -633,7 +645,7 @@ flexHammerThread(void* pArg)
 
 
 int
-readArguments (int argc, const char** argv)
+readArguments (int argc, char** argv)
 {
   int i = 1;
   
